# -*- coding: utf-8 -*-
"""
method to open correlator.com .sin files
"""
import os
import csv
import numpy as np


def openSIN(dirname, filename):
    """ Read data from a .SIN file, usually created by
        the software using correlators from correlator.com.

            FLXA
            Version= 1d

            [Parameters]
            ...
            Mode= Single Auto
            ...

            [CorrelationFunction]
            1.562500e-09	0.000000e+00
            3.125000e-09	0.000000e+00
            4.687500e-09	0.000000e+00
            ...
            1.887435e+01	1.000030e+00
            1.929378e+01	1.000141e+00
            1.971321e+01	9.999908e-01
            2.013264e+01	9.996810e-01
            2.055207e+01	1.000047e+00
            2.097150e+01	9.999675e-01
            2.139093e+01	9.999591e-01
            2.181036e+01	1.000414e+00
            2.222979e+01	1.000129e+00
            2.264922e+01	9.999285e-01
            2.306865e+01	1.000077e+00
            ...
            3.959419e+02	0.000000e+00
            4.026528e+02	0.000000e+00
            4.093637e+02	0.000000e+00
            4.160746e+02	0.000000e+00
            4.227854e+02	0.000000e+00
            4.294963e+02	0.000000e+00

            [RawCorrelationFunction]
            ...

            [IntensityHistory]
            TraceNumber= 458
            0.000000	9.628296e+03	9.670258e+03
            0.262144	1.001358e+04	9.971619e+03
            0.524288	9.540558e+03	9.548188e+03
            0.786432	9.048462e+03	9.010315e+03
            1.048576	8.815766e+03	8.819580e+03
            1.310720	8.827210e+03	8.861542e+03
            1.572864	9.201050e+03	9.185791e+03
            1.835008	9.124756e+03	9.124756e+03
            2.097152	9.059906e+03	9.029389e+03
            ...

        1. We are interested in the "[CorrelationFunction]" section,
        where the first column denotes tau in seconds and the second row the
        correlation signal. Values are separated by a tabulator "\t".
        We do not import anything from the "[Parameters]" section.
        We have to subtract "1" from the correlation function, since it
        is a correlation function that converges to "1" and not to "0".

        2. We are also interested in the "[IntensityHistory]" section.
        If we are only interested in autocorrelation functions: An email
        from Jixiang Zhu - Correlator.com (2012-01-22) said, that
        "For autocorrelation mode, the 2nd and 3 column represent the same
        intensity series with slight delay.  Therefore, they are statistically
        the same but numerically different."
        It is therefore perfectly fine to just use the 2nd column.

        Different acquisition modes:
        Mode            [CorrelationFunction]               [IntensityHistory]
        Single Auto     2 Colums (tau,AC)                   1 significant
        Single Cross    2 Colums (tau,CC)                   2
        Dual Auto       3 Colums (tau,AC1,AC2)              2
        Dual Cross      3 Colums (tau,CC12,CC21)            2
        Quad            5 Colums (tau,AC1,AC2,CC12,CC21)    2

        Returns:
        [0]:
         N arrays with tuples containing two elements:
         1st: tau in ms
         2nd: corresponding correlation signal
        [1]:
         N Intensity traces:
         1st: time in ms
         2nd: Trace in kHz
        [2]: 
         A list with N elements, indicating, how many correlation
         curves we are importing.
    """
    openfile = open(os.path.join(dirname, filename), 'r')
    Alldata = openfile.readlines()
    # Find out where the correlation function and trace are
    for i in np.arange(len(Alldata)):
        if Alldata[i][0:4] == "Mode":
         
            Mode = Alldata[i].split("=")[1].strip()
        if Alldata[i][0:21] == "[CorrelationFunction]":
            StartC = i+1
        if Alldata[i][0:24] == "[RawCorrelationFunction]":
            EndC = i-2
        if Alldata[i][0:18] == "[IntensityHistory]":
            # plus 2, because theres a line with the trace length
            StartT = i+2 
        if Alldata[i][0:11] == "[Histogram]":
            EndT = i-2
    curvelist = list()
    correlations = list()
    traces = list()
    # Get the correlation function
    Truedata = Alldata.__getslice__(StartC, EndC)
    timefactor = 1000 # because we want ms instead of s
    readcorr = csv.reader(Truedata, delimiter='\t')
    # Trace
    # Trace is stored in three columns
    # 1st column: time [s]
    # 2nd column: trace [Hz] 
    # 3rd column: trace [Hz] - Single Auto: equivalent to 2nd
    # Get the trace
    Tracedata = Alldata.__getslice__(StartT, EndT)
    # timefactor = 1000 # because we want ms instead of s
    timedivfac = 1000 # because we want kHz instead of Hz
    readtrace = csv.reader(Tracedata, delimiter='\t')
    openfile.close()
    # Process all Data:
    if Mode == "Single Auto":
        curvelist.append("AC")
        corrdata = list()
        for row in readcorr:
            # tau in ms, corr-function minus "1"
            corrdata.append((np.float(row[0])*timefactor, np.float(row[1])-1))
        correlations.append(np.array(corrdata))
        trace = list()
        for row in readtrace:
            # tau in ms, corr-function minus "1"
            trace.append((np.float(row[0])*timefactor,
                         np.float(row[1])/timedivfac))
        traces.append(np.array(trace))
    elif Mode == "Single Cross":
        curvelist.append("CC")
        corrdata = list()
        for row in readcorr:
            # tau in ms, corr-function minus "1"
            corrdata.append((np.float(row[0])*timefactor, np.float(row[1])-1))
        correlations.append(np.array(corrdata))
        trace1 = list()
        trace2 = list()
        for row in readtrace:
            # tau in ms, corr-function minus "1"
            trace1.append((np.float(row[0])*timefactor,
                           np.float(row[1])/timedivfac))
            trace2.append((np.float(row[0])*timefactor,
                           np.float(row[2])/timedivfac))
        traces.append([np.array(trace1), np.array(trace2)])
    elif Mode == "Dual Auto":
        curvelist.append("AC1")
        curvelist.append("AC2")
        corrdata1 = list()
        corrdata2 = list()
        for row in readcorr:
            # tau in ms, corr-function minus "1"
            corrdata1.append((np.float(row[0])*timefactor, np.float(row[1])-1))
            corrdata2.append((np.float(row[0])*timefactor, np.float(row[2])-1))
        correlations.append(np.array(corrdata1))
        correlations.append(np.array(corrdata2))
        trace1 = list()
        trace2 = list()
        for row in readtrace:
            # tau in ms, corr-function minus "1"
            trace1.append((np.float(row[0])*timefactor,
                           np.float(row[1])/timedivfac))
            trace2.append((np.float(row[0])*timefactor,
                           np.float(row[2])/timedivfac))
        traces.append(np.array(trace1))
        traces.append(np.array(trace2))
    elif Mode == "Dual Cross":
        curvelist.append("CC12")
        curvelist.append("CC21")
        corrdata1 = list()
        corrdata2 = list()
        for row in readcorr:
            # tau in ms, corr-function minus "1"
            corrdata1.append((np.float(row[0])*timefactor, np.float(row[1])-1))
            corrdata2.append((np.float(row[0])*timefactor, np.float(row[2])-1))
        correlations.append(np.array(corrdata1))
        correlations.append(np.array(corrdata2))
        trace1 = list()
        trace2 = list()
        for row in readtrace:
            # tau in ms, corr-function minus "1"
            trace1.append((np.float(row[0])*timefactor,
                           np.float(row[1])/timedivfac))
            trace2.append((np.float(row[0])*timefactor,
                           np.float(row[2])/timedivfac))
        traces.append([np.array(trace1), np.array(trace2)])
        traces.append([np.array(trace1), np.array(trace2)])
    elif Mode == "Quad":
        curvelist.append("AC1")
        curvelist.append("AC2")
        curvelist.append("CC12")
        curvelist.append("CC21")
        corrdata1 = list()
        corrdata2 = list()
        corrdata12 = list()
        corrdata21 = list()
        for row in readcorr:
            # tau in ms, corr-function minus "1"
            corrdata1.append((np.float(row[0])*timefactor, np.float(row[1])-1))
            corrdata2.append((np.float(row[0])*timefactor, np.float(row[2])-1))
            corrdata12.append((np.float(row[0])*timefactor, np.float(row[3])-1))
            corrdata21.append((np.float(row[0])*timefactor, np.float(row[4])-1))
        correlations.append(np.array(corrdata1))
        correlations.append(np.array(corrdata2))
        correlations.append(np.array(corrdata12))
        correlations.append(np.array(corrdata21))
        trace1 = list()
        trace2 = list()
        for row in readtrace:
            # tau in ms, corr-function minus "1"
            trace1.append((np.float(row[0])*timefactor,
                           np.float(row[1])/timedivfac))
            trace2.append((np.float(row[0])*timefactor,
                           np.float(row[2])/timedivfac))
        traces.append(np.array(trace1))
        traces.append(np.array(trace2))
        traces.append([np.array(trace1), np.array(trace2)])
        traces.append([np.array(trace1), np.array(trace2)])
    else:
        # We assume that we have a mode like this:
        # Mode= 2 3 3 2 0 1 1 0
        raise NotImplemented("This format is not yet implemented (issue #135).")
        # TODO:
        # - write test function for .sin correlator files
        # - load correlation and traces from sin files in 2D arrays
        # -> use this if, elif, elif, else loop to only assign curves
        #    (to improve code quality)
        
    dictionary = dict()
    dictionary["Correlation"] = correlations
    dictionary["Trace"] = traces
    dictionary["Type"] = curvelist
    filelist = list()
    for i in curvelist:
        filelist.append(filename)
    dictionary["Filename"] = filelist
    return dictionary



# -*- coding: utf-8 -*-
"""
PyCorrFit

Module tools - globalfit
Perform global fitting on pages which share parameters.
"""


import wx
import numpy as np

from .. import misc
from ..fcs_data_set import Fit

# Menu entry name
MENUINFO = ["&Global fitting",
            "Interconnect parameters from different measurements."]

class GlobalFit(wx.Frame):
    # This tool is derived from a wx.frame.
    def __init__(self, parent):
        # Define a unique name that identifies this tool
        # Do not change this value. It is important for the Overlay tool
        # (selectcurves.py, *Wrapper_Tools*).
        self.MyName="GLOBALFIT"
        # parent is the main frame of PyCorrFit
        self.parent = parent
        # Get the window positioning correctly
        pos = self.parent.GetPosition()
        pos = (pos[0]+100, pos[1]+100)
        wx.Frame.__init__(self, parent=self.parent, title="Gobal fitting",
                 pos=pos, style=wx.DEFAULT_FRAME_STYLE|wx.FRAME_FLOAT_ON_PARENT)
        ## MYID
        # This ID is given by the parent for an instance of this class
        self.MyID = None
        # Page - the currently active page of the notebook.
        self.Page = self.parent.notebook.GetCurrentPage()
        ## Content
        self.panel = wx.Panel(self)
        self.topSizer = wx.BoxSizer(wx.VERTICAL)
        textinit = """Fitting of multiple data sets with different models.
Parameter names have to match. Select pages (e.g. 1,3-5,7),
check parameters on each page and start 'Global fit'. 
"""
        self.topSizer.Add(wx.StaticText(self.panel, label=textinit))
        ## Page selection
        self.WXTextPages = wx.TextCtrl(self.panel, value="", size=(330,-1))
        # Set initial value in text control
        pagenumlist = list()
        for i in np.arange(self.parent.notebook.GetPageCount()):
            Page = self.parent.notebook.GetPage(i)
            pagenumlist.append(int(filter(lambda x: x.isdigit(), Page.counter)))
        valstring=misc.parsePagenum2String(pagenumlist)
        self.WXTextPages.SetValue(valstring)
        self.topSizer.Add(self.WXTextPages)
        ## Button
        btnfit = wx.Button(self.panel, wx.ID_ANY, 'Global fit')
        # Binds the button to the function - close the tool
        self.Bind(wx.EVT_BUTTON, self.OnFit, btnfit)
        self.topSizer.Add(btnfit)
        self.panel.SetSizer(self.topSizer)
        self.topSizer.Fit(self)
        self.SetMinSize(self.topSizer.GetMinSizeTuple())
        self.OnPageChanged(self.Page)
        # Icon
        if parent.MainIcon is not None:
            wx.Frame.SetIcon(self, parent.MainIcon)
        self.Show(True)

    def OnClose(self, event=None):
        # This is a necessary function for PyCorrFit.
        # Do not change it.
        self.parent.toolmenu.Check(self.MyID, False)
        self.parent.ToolsOpen.__delitem__(self.MyID)
        self.Destroy()

    def OnFit(self, e=None):
        # process a string like this: "1,2,4-9,10"
        strFull = self.WXTextPages.GetValue()
        PageNumbers = misc.parseString2Pagenum(self, strFull)
        global_pages = list()
        if PageNumbers is None:
            # Something went wrong and parseString2Pagenum already displayed
            # an error message.
            return
        ## Get the correlations
        corrs = list()
        for i in np.arange(self.parent.notebook.GetPageCount()):
            Page = self.parent.notebook.GetPage(i)
            corr = Page.corr
            j = filter(lambda x: x.isdigit(), Page.counter)
            if int(j) in PageNumbers:
                if corr.correlation is not None:
                    Page.apply_parameters()
                    corrs.append(corr)
                    global_pages.append(int(j))
                else:
                    print "No experimental data in page #"+j+"!"

        if len(corrs) == 0:
            return
        
        # Perform fit
        fitter = Fit(corrs, global_fit=True)
        fit_parm_names = [f.split()[0] for f in fitter.fit_parm_names]

        # update fit results
        for corr in corrs:
            corr.fit_results["global parms"] = u", ".join(fit_parm_names)
            corr.fit_results["global pages"] = u", ".join([str(g) for g in global_pages])
        
        # Plot resutls
        for i in np.arange(self.parent.notebook.GetPageCount()):
            Page = self.parent.notebook.GetPage(i)
            j = filter(lambda x: x.isdigit(), Page.counter)
            if int(j) in global_pages:
                Page.apply_parameters_reverse()
                Page.PlotAll()

    def OnPageChanged(self, page, trigger=None):
        """
            This function is called, when something in the panel
            changes. The variable `trigger` is used to prevent this
            function from being executed to save stall time of the user.
            Forr a list of possible triggers, see the doc string of
            `tools`.
        """
        # When parent changes
        # This is a necessary function for PyCorrFit.
        # This is stuff that should be done when the active page
        # of the notebook changes.
        if trigger in ["parm_batch", "fit_batch", "page_add_batch"]:
            return
        if self.parent.notebook.GetPageCount() == 0:
            self.panel.Disable()
            return
        self.panel.Enable()
        self.Page = page


    def SetPageNumbers(self, pagestring):
        self.WXTextPages.SetValue(pagestring)



# -*- coding: utf-8 -*-
""" Wrapper for Loading PicoQuant .pt3 data files

Wraps around FCS_point_correlator by Dominic Waithe
https://github.com/dwaithe/FCS_point_correlator
"""
import numpy as np
import os
from .read_pt3_scripts.correlation_objects import picoObject

from . import util

class ParameterClass():
    """Stores parameters for correlation """
    def __init__(self):
        #Where the data is stored.
        self.data = []
        self.objectRef =[]
        self.subObjectRef =[]
        self.colors = ['blue','green','red','cyan','magenta','yellow','black']
        self.numOfLoaded = 0
        self.NcascStart = 0
        self.NcascEnd = 25
        self.Nsub = 6
        self.winInt = 10
        self.photonCountBin = 25
        

def getTrace(picoObject, number):
    """
    Extracts trace `number` from a `picoObject`.
    
    Parameters
    ----------
    picoObject: instance of picoObject
        The data retreived from a pt3 file
    number:
        The id of the trace, can be 1 or 2.
    """
    
    attrint = "timeSeries{}".format(number)
    attrtime = "timeSeriesScale{}".format(number)

    # binned photon counts
    intensity = np.array(getattr(picoObject, attrint))
    # Time in ms for each bin
    time = np.array(getattr(picoObject, attrtime))
    # time delta
    deltat = np.abs(time[2]-time[1])
    
    trace = np.zeros((intensity.shape[0],2))
    trace[:,0] = time # ms
    trace[:,1] = intensity / deltat # kHz
    
    # If the trace is too big. Wee need to bin it.
    newtrace = util.downsample_trace(trace)
    
    return newtrace
    


def openPT3(dirname, filename):
    """ Retreive correlation curves from PicoQuant data files 
    
    This function is a wrapper around the PicoQuant capability of
    FCS_Viewer by Dominic Waithe.
    """
    par_obj = ParameterClass()
    
    pt3file = picoObject(os.path.join(dirname, filename), par_obj, None)

    po = pt3file

    auto = po.autoNorm
    # lag time [ms]
    autotime = po.autotime.reshape(-1)

    corrlist = list()
    typelist = list()
    tracelist = list()
    # Some data points are zero for some reason
    id1 = np.where(autotime!=0)


    # AC0 - autocorrelation CH0
    corrac0 = auto[:,0,0]    
    if np.sum(np.abs(corrac0[id1])) != 0:
        typelist.append("AC0")
        # autotime,auto[:,0,0]
        corrlist.append(np.hstack( (autotime[id1].reshape(-1,1),
                                    corrac0[id1].reshape(-1,1)) ))
        tracelist.append([getTrace(po, 1)])
    
    # AC1 - autocorrelation CH1
    corrac1 = auto[:,1,1]
    if np.sum(np.abs(corrac1[id1])) != 0:
        typelist.append("AC1")
        # autotime,auto[:,1,1]
        corrlist.append(np.hstack( (autotime[id1].reshape(-1,1),
                                    corrac1[id1].reshape(-1,1)) ))
        tracelist.append([getTrace(po, 2)])
    
    # CC01 - Cross-Correlation CH0-CH1
    corrcc01 = auto[:,0,1]
    if np.sum(np.abs(corrcc01[id1])) != 0:
        typelist.append("CC01")
        # autotime,auto[:,0,1]
        corrlist.append(np.hstack( (autotime[id1].reshape(-1,1),
                                    corrcc01[id1].reshape(-1,1)) ))
        tracelist.append([getTrace(po, 1), getTrace(po, 2)])
    
    # CC10 - Cross-Correlation CH1-CH0
    corrcc10 = auto[:,1,0]
    if np.sum(np.abs(corrcc10[id1])) != 0:
        typelist.append("CC10")
        # autotime,auto[:,1,0]
        corrlist.append(np.hstack( (autotime[id1].reshape(-1,1),
                                    corrcc10[id1].reshape(-1,1)) ))
        tracelist.append([getTrace(po, 1), getTrace(po, 2)])

    filelist = [filename] * len(typelist)

    dictionary = dict()
    dictionary["Correlation"] = corrlist
    dictionary["Trace"] = tracelist
    dictionary["Type"] = typelist
    dictionary["Filename"] = filelist

    return dictionary



import numpy as np
import os, sys
from correlation_methods import *
from import_methods import *
import time
from fitting_methods import equation_
#from lmfit import minimize, Parameters,report_fit,report_errors, fit_report
import csv
import copy


"""FCS Bulk Correlation Software

	Copyright (C) 2015  Dominic Waithe

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License along
	with this program; if not, write to the Free Software Foundation, Inc.,
	51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""

class picoObject():
	#This is the class which holds the .pt3 data and parameters
	def __init__(self,filepath, par_obj,fit_obj):
	
		#parameter object and fit object. If 
		self.par_obj = par_obj
		self.fit_obj = fit_obj
		self.type = 'mainObject'
		
		#self.PIE = 0
		self.filepath = str(filepath)
		self.nameAndExt = os.path.basename(self.filepath).split('.')
		self.name = self.nameAndExt[0]
		self.ext = self.nameAndExt[-1]

		self.par_obj.data.append(filepath);
		self.par_obj.objectRef.append(self)
		
		#Imports pt3 file format to object.
		self.unqID = self.par_obj.numOfLoaded
		
		#For fitting.
		self.objId1 = None
		self.objId2 = None
		self.objId3 = None
		self.objId4 = None
		self.processData();
		
		self.plotOn = True;


	def processData(self):

		self.NcascStart = self.par_obj.NcascStart
		self.NcascEnd = self.par_obj.NcascEnd
		self.Nsub = self.par_obj.Nsub
		self.winInt = self.par_obj.winInt
		self.photonCountBin = self.par_obj.photonCountBin
		
		#File import
		
		if self.ext == 'pt3':
			self.subChanArr, self.trueTimeArr, self.dTimeArr,self.resolution = pt3import(self.filepath)
		if self.ext == 'csv':
			self.subChanArr, self.trueTimeArr, self.dTimeArr,self.resolution = csvimport(self.filepath)
			#If the file is empty.
			if self.subChanArr == None:
				#Undoes any preparation of resource.
				self.par_obj.data.pop(-1);
				self.par_obj.objectRef.pop(-1)
				self.exit = True
				self.par_obj.image_status_text.showMessage("Your sample is not in the correct format.")
				self.par_obj.fit_obj.app.processEvents()
				return

					
		
		#Colour assigned to file.
		self.color = self.par_obj.colors[self.unqID % len(self.par_obj.colors)]

		#How many channels there are in the files.
		self.numOfCH =  np.unique(np.array(self.subChanArr)).__len__()-1 #Minus 1 because not interested in channel 15.
		
		#Finds the numbers which address the channels.
		self.ch_present = np.unique(np.array(self.subChanArr[0:100]))

		#Calculates decay function for both channels.
		self.photonDecayCh1,self.decayScale1 = delayTime2bin(np.array(self.dTimeArr),np.array(self.subChanArr),self.ch_present[0],self.winInt)
		
		if self.numOfCH ==  2:
			self.photonDecayCh2,self.decayScale2 = delayTime2bin(np.array(self.dTimeArr),np.array(self.subChanArr),self.ch_present[1],self.winInt)

		#Time series of photon counts. For visualisation.
		self.timeSeries1,self.timeSeriesScale1 = delayTime2bin(np.array(self.trueTimeArr)/1000000,np.array(self.subChanArr),self.ch_present[0],self.photonCountBin)
		
		unit = self.timeSeriesScale1[-1]/self.timeSeriesScale1.__len__()
		
		#Converts to counts per 
		self.kcount_CH1 = np.average(self.timeSeries1)

		raw_count = np.average(self.timeSeries1) #This is the unnormalised intensity count for int_time duration (the first moment)
		var_count = np.var(self.timeSeries1)

		self.brightnessNandBCH0=(((var_count -raw_count)/(raw_count))/(float(unit)))
		if (var_count-raw_count) == 0:
			self.numberNandBCH0 =0
		else:
			self.numberNandBCH0 = (raw_count**2/(var_count-raw_count))
		


		if self.numOfCH ==  2:

			self.timeSeries2,self.timeSeriesScale2 = delayTime2bin(np.array(self.trueTimeArr)/1000000,np.array(self.subChanArr),self.ch_present[1],self.photonCountBin)
			unit = self.timeSeriesScale2[-1]/self.timeSeriesScale2.__len__()
			self.kcount_CH2 = np.average(self.timeSeries2)
			raw_count = np.average(self.timeSeries2) #This is the unnormalised intensity count for int_time duration (the first moment)
			var_count = np.var(self.timeSeries2)
			self.brightnessNandBCH1= (((var_count -raw_count)/(raw_count))/(float(unit)))
			if (var_count-raw_count) == 0:
				self.numberNandBCH1 =0
			else:
				self.numberNandBCH1 = (raw_count**2/(var_count-raw_count))


		
		#Calculates the Auto and Cross-correlation functions.
		self.crossAndAuto(np.array(self.trueTimeArr),np.array(self.subChanArr))
		
		if self.fit_obj != None:
			#If fit object provided then creates fit objects.
			if self.objId1 == None:
				corrObj= corrObject(self.filepath,self.fit_obj);
				self.objId1 = corrObj.objId
				self.fit_obj.objIdArr.append(corrObj.objId)
				self.objId1.param = copy.deepcopy(self.fit_obj.def_param)
				self.objId1.name = self.name+'_CH0_Auto_Corr'
				self.objId1.ch_type = 0 #channel 0 Auto
				self.objId1.siblings = None
				self.objId1.prepare_for_fit()
				self.objId1.kcount = self.kcount_CH1
				
			self.objId1.autoNorm = np.array(self.autoNorm[:,0,0]).reshape(-1)
			self.objId1.autotime = np.array(self.autotime).reshape(-1)
			self.objId1.param = copy.deepcopy(self.fit_obj.def_param)
			
			
			if self.numOfCH ==  2:
				if self.objId3 == None:
					corrObj= corrObject(self.filepath,self.fit_obj);
					self.objId3 = corrObj.objId
					self.objId3.param = copy.deepcopy(self.fit_obj.def_param)
					self.fit_obj.objIdArr.append(corrObj.objId)
					self.objId3.name = self.name+'_CH1_Auto_Corr'
					self.objId3.ch_type = 1 #channel 1 Auto
					self.objId3.siblings = None
					self.objId3.prepare_for_fit()
					self.objId3.kcount = self.kcount_CH2
					
				self.objId3.autoNorm = np.array(self.autoNorm[:,1,1]).reshape(-1)
				self.objId3.autotime = np.array(self.autotime).reshape(-1)
				self.objId3.param = copy.deepcopy(self.fit_obj.def_param)
				
				if self.objId2 == None:
					corrObj= corrObject(self.filepath,self.fit_obj);
					self.objId2 = corrObj.objId
					self.objId2.param = copy.deepcopy(self.fit_obj.def_param)
					self.fit_obj.objIdArr.append(corrObj.objId)
					self.objId2.name = self.name+'_CH01_Cross_Corr'
					self.objId2.ch_type = 2 #01cross
					self.objId2.siblings = None
					self.objId2.prepare_for_fit()
					
				self.objId2.autoNorm = np.array(self.autoNorm[:,0,1]).reshape(-1)
				self.objId2.autotime = np.array(self.autotime).reshape(-1)
				self.objId2.param = copy.deepcopy(self.fit_obj.def_param)
				

				if self.objId4 == None:
					corrObj= corrObject(self.filepath,self.fit_obj);
					self.objId4 = corrObj.objId
					self.objId4.param = copy.deepcopy(self.fit_obj.def_param)
					self.fit_obj.objIdArr.append(corrObj.objId)
					self.objId4.name = self.name+'_CH10_Cross_Corr'
					self.objId4.ch_type = 3 #10cross
					self.objId4.siblings = None
					self.objId4.prepare_for_fit()
					
				self.objId4.autoNorm = np.array(self.autoNorm[:,1,0]).reshape(-1)
				self.objId4.autotime = np.array(self.autotime).reshape(-1)
				self.objId4.param = copy.deepcopy(self.fit_obj.def_param)
				
			self.fit_obj.fill_series_list()
		self.dTimeMin = 0
		self.dTimeMax = np.max(self.dTimeArr)
		self.subDTimeMin = self.dTimeMin
		self.subDTimeMax = self.dTimeMax
		self.exit = False
		del self.subChanArr 
		del self.trueTimeArr 
		del self.dTimeArr
	def crossAndAuto(self,trueTimeArr,subChanArr):
		#For each channel we loop through and find only those in the correct time gate.
		#We only want photons in channel 1 or two.
		y = trueTimeArr[subChanArr < 3]
		validPhotons = subChanArr[subChanArr < 3 ]


		#Creates boolean for photon events in either channel.
		num = np.zeros((validPhotons.shape[0],2))
		num[:,0] = (np.array([np.array(validPhotons) ==self.ch_present[0]])).astype(np.int32)
		if self.numOfCH ==2:
			num[:,1] = (np.array([np.array(validPhotons) ==self.ch_present[1]])).astype(np.int32)


		self.count0 = np.sum(num[:,0]) 
		self.count1 = np.sum(num[:,1])

		t1 = time.time()
		auto, self.autotime = tttr2xfcs(y,num,self.NcascStart,self.NcascEnd, self.Nsub)
		t2 = time.time()
		
		

		#Normalisation of the TCSPC data:
		maxY = np.ceil(max(self.trueTimeArr))
		self.autoNorm = np.zeros((auto.shape))
		self.autoNorm[:,0,0] = ((auto[:,0,0]*maxY)/(self.count0*self.count0))-1
		
		if self.numOfCH ==  2:
			self.autoNorm[:,1,1] = ((auto[:,1,1]*maxY)/(self.count1*self.count1))-1
			self.autoNorm[:,1,0] = ((auto[:,1,0]*maxY)/(self.count1*self.count0))-1
			self.autoNorm[:,0,1] = ((auto[:,0,1]*maxY)/(self.count0*self.count1))-1
			

		#Normalisaation of the decay functions.
		self.photonDecayCh1Min = self.photonDecayCh1-np.min(self.photonDecayCh1)
		self.photonDecayCh1Norm = self.photonDecayCh1Min/np.max(self.photonDecayCh1Min)
		
		
		if self.numOfCH ==  2:
			self.photonDecayCh2Min = self.photonDecayCh2-np.min(self.photonDecayCh2)
			self.photonDecayCh2Norm = self.photonDecayCh2Min/np.max(self.photonDecayCh2Min)
		
		return 
   

	
	
class subPicoObject():
	def __init__(self,parentId,xmin,xmax,TGid,par_obj):
		#Binning window for decay function
		self.TGid = TGid
		#Parameters for auto-correlation and cross-correlation.
		self.parentId = parentId
		self.par_obj = par_obj
		self.NcascStart = self.parentId.NcascStart
		self.NcascEnd = self.parentId.NcascEnd
		self.Nsub = self.parentId.Nsub
		self.fit_obj = self.parentId.fit_obj
		self.ext = self.parentId.ext
		
		self.type = 'subObject'
		#Appends the object to the subObject register.
		self.par_obj.subObjectRef.append(self)
		self.unqID = self.par_obj.subNum
		self.parentUnqID = self.parentId.unqID
		#self.chanArr = parentObj.chanArr
		#self.trueTimeArr = self.parentId.trueTimeArr
		#self.dTimeArr = self.parentId.dTimeArr
		self.color = self.parentId.color
		self.numOfCH = self.parentId.numOfCH
		self.ch_present = self.parentId.ch_present
		self.photonCountBin = self.par_obj.photonCountBin

		self.filepath = str(self.parentId.filepath)
		self.xmin = xmin
		self.xmax = xmax

		self.nameAndExt = os.path.basename(self.filepath).split('.')
		self.name = 'TG-'+str(self.unqID)+'-xmin_'+str(round(xmin,0))+'-xmax_'+str(round(xmax,0))+'-'+self.nameAndExt[0]

		self.objId1 = None
		self.objId2 = None
		self.objId3 = None
		self.objId4 = None
		self.processData();
		self.plotOn = True
		
		
	def processData(self):
		self.NcascStart= self.par_obj.NcascStart
		self.NcascEnd= self.par_obj.NcascEnd
		self.Nsub = self.par_obj.Nsub
		self.winInt = self.par_obj.winInt
		
		
		#self.subChanArr, self.trueTimeArr, self.dTimeArr,self.resolution = pt3import(self.filepath)
		if self.ext == 'pt3':
			self.subChanArr, self.trueTimeArr, self.dTimeArr,self.resolution = pt3import(self.filepath)
		if self.ext == 'csv':
			self.subChanArr, self.trueTimeArr, self.dTimeArr,self.resolution = csvimport(self.filepath)
			#If the file is empty.
			#if self.subChanArr == None:
				#Undoes any preparation of resource.
			#    self.par_obj.subObjectRef.pop(-1)
				#self.exit = True
			#    return


		self.subArrayGeneration(self.xmin,self.xmax)
		
		self.dTimeMin = self.parentId.dTimeMin
		self.dTimeMax = self.parentId.dTimeMax
		self.subDTimeMin = self.dTimeMin
		self.subDTimeMax = self.dTimeMax

	   #Time series of photon counts. For visualisation.
		self.timeSeries1,self.timeSeriesScale1 = delayTime2bin(np.array(self.trueTimeArr)/1000000,np.array(self.subChanArr),self.ch_present[0],self.photonCountBin)
		
		unit = self.timeSeriesScale1[-1]/self.timeSeriesScale1.__len__()
		self.kcount_CH1 = np.average(self.timeSeries1)
		if self.numOfCH ==  2:

			self.timeSeries2,self.timeSeriesScale2 = delayTime2bin(np.array(self.trueTimeArr)/1000000,np.array(self.subChanArr),self.ch_present[1],self.photonCountBin)
			unit = self.timeSeriesScale2[-1]/self.timeSeriesScale2.__len__()
			self.kcount_CH2 = np.average(self.timeSeries2)
		

		
		#Adds names to the fit function for later fitting.
		if self.objId1 == None:
			corrObj= corrObject(self.filepath,self.fit_obj);
			self.objId1 = corrObj.objId
			self.fit_obj.objIdArr.append(corrObj.objId)
			self.objId1.param = copy.deepcopy(self.fit_obj.def_param)
			self.objId1.name = self.name+'_CH0_Auto_Corr'
			self.objId1.ch_type = 0 #channel 0 Auto
			self.objId1.siblings = None
			self.objId1.prepare_for_fit()
			
			self.objId1.kcount = self.kcount_CH1
		self.objId1.autoNorm = np.array(self.autoNorm[:,0,0]).reshape(-1)
		self.objId1.autotime = np.array(self.autotime).reshape(-1)
		self.objId1.param = copy.deepcopy(self.fit_obj.def_param)
		
		
		if self.numOfCH == 2:
			if self.objId3 == None:
				corrObj= corrObject(self.filepath,self.fit_obj);
				self.objId3 = corrObj.objId
				self.fit_obj.objIdArr.append(corrObj.objId)
				self.objId3.param = copy.deepcopy(self.fit_obj.def_param)
				self.objId3.name = self.name+'_CH1_Auto_Corr'
				self.objId3.ch_type = 1 #channel 1 Auto
				self.objId3.siblings = None
				self.objId3.prepare_for_fit()
				self.objId3.kcount = self.kcount_CH2
				
			self.objId3.autoNorm = np.array(self.autoNorm[:,1,1]).reshape(-1)
			self.objId3.autotime = np.array(self.autotime).reshape(-1)
			self.objId3.param = copy.deepcopy(self.fit_obj.def_param)
			if self.objId2 == None:
				corrObj= corrObject(self.filepath,self.fit_obj);
				self.objId2 = corrObj.objId
				self.objId2.param = copy.deepcopy(self.fit_obj.def_param)
				self.fit_obj.objIdArr.append(corrObj.objId)
				self.objId2.name = self.name+'_CH01_Cross_Corr'
				self.objId2.ch_type = 2 #channel 01 Cross
				self.objId2.siblings = None
				self.objId2.prepare_for_fit()
				
			self.objId2.autoNorm = np.array(self.autoNorm[:,0,1]).reshape(-1)
			self.objId2.autotime = np.array(self.autotime).reshape(-1)
			self.objId2.param = copy.deepcopy(self.fit_obj.def_param)
			if self.objId4 == None:
				corrObj= corrObject(self.filepath,self.fit_obj);
				self.objId4 = corrObj.objId
				self.objId4.param = copy.deepcopy(self.fit_obj.def_param)
				self.fit_obj.objIdArr.append(corrObj.objId)
				self.objId4.name = self.name+'_CH10_Cross_Corr'
				self.objId4.ch_type = 3 #channel 10 Cross
				self.objId4.siblings = None
				self.objId4.prepare_for_fit()
				
			self.objId4.autoNorm = np.array(self.autoNorm[:,1,0]).reshape(-1)
			self.objId4.autotime = np.array(self.autotime).reshape(-1)
			
			
		
		self.fit_obj.fill_series_list()  
		del self.subChanArr 
		del self.trueTimeArr 
		del self.dTimeArr 
	


	def subArrayGeneration(self,xmin,xmax):
		if(xmax<xmin):
			xmin1 = xmin
			xmin = xmax
			xmax = xmin1
		#self.subChanArr = np.array(self.chanArr)
		#Finds those photons which arrive above certain time or below certain time.
		photonInd = np.logical_and(self.dTimeArr>=xmin, self.dTimeArr<=xmax).astype(np.bool)
		
		self.subChanArr[np.invert(photonInd).astype(np.bool)] = 16
		
		self.crossAndAuto()

		return
	def crossAndAuto(self):
		#We only want photons in channel 1 or two.
		validPhotons = self.subChanArr[self.subChanArr < 3]
		y = self.trueTimeArr[self.subChanArr < 3]
		#Creates boolean for photon events in either channel.
		num = np.zeros((validPhotons.shape[0],2))
		num[:,0] = (np.array([np.array(validPhotons) ==self.ch_present[0]])).astype(np.int)
		if self.numOfCH == 2:
			num[:,1] = (np.array([np.array(validPhotons) ==self.ch_present[1]])).astype(np.int)

		self.count0 = np.sum(num[:,0]) 
		self.count1 = np.sum(num[:,1]) 
		#Function which calculates auto-correlation and cross-correlation.



		auto, self.autotime = tttr2xfcs(y,num,self.NcascStart,self.NcascEnd, self.Nsub)

		maxY = np.ceil(max(self.trueTimeArr))
		self.autoNorm = np.zeros((auto.shape))
		self.autoNorm[:,0,0] = ((auto[:,0,0]*maxY)/(self.count0*self.count0))-1
		if self.numOfCH ==2:
			self.autoNorm[:,1,1] = ((auto[:,1,1]*maxY)/(self.count1*self.count1))-1
			self.autoNorm[:,1,0] = ((auto[:,1,0]*maxY)/(self.count1*self.count0))-1
			self.autoNorm[:,0,1] = ((auto[:,0,1]*maxY)/(self.count0*self.count1))-1

		return 

class corrObject():
	def __init__(self,filepath,parentFn):
		#the container for the object.
		self.parentFn = parentFn
		self.type = 'corrObject'
		self.filepath = str(filepath)
		self.nameAndExt = os.path.basename(self.filepath).split('.')
		self.name = self.nameAndExt[0]
		self.ext = self.nameAndExt[-1]
		self.autoNorm=[]
		self.autotime=[]
		self.model_autoNorm =[]
		self.model_autotime = []
		self.datalen= []
		self.objId = self;
		self.param = []
		self.goodFit = True
		self.fitted = False
		self.checked = False
		self.clicked = False
		self.toFit = False
		self.kcount = None
		self.filter = False
	   
	def prepare_for_fit(self):
		if self.parentFn.ch_check_ch0.isChecked() == True and self.ch_type == 0:
			self.toFit = True
		if self.parentFn.ch_check_ch1.isChecked() == True and self.ch_type == 1:
			self.toFit = True
			
		if self.parentFn.ch_check_ch01.isChecked() == True and self.ch_type == 2:
			self.toFit = True
		if self.parentFn.ch_check_ch10.isChecked() == True and self.ch_type == 3:
			self.toFit = True
		#self.parentFn.modelFitSel.clear()
		#for objId in self.parentFn.objIdArr:
		 #   if objId.toFit == True:
		  #      self.parentFn.modelFitSel.addItem(objId.name)
		self.parentFn.updateFitList()
	def residual(self, param, x, data,options):
	
		A = equation_(param, x,options)
		residuals = data-A
		return residuals
	def fitToParameters(self):
		#self.parentFn.updateParamFirst()
		#self.parentFn.updateTableFirst()
		#self.parentFn.updateParamFirst()
		

		#Populate param for lmfit.
		param = Parameters()
		#self.def_param.add('A1', value=1.0, min=0,max=1.0, vary=False)
		for art in self.param:
			
			if self.param[art]['to_show'] == True:
				
				param.add(art, value=float(self.param[art]['value']), min=float(self.param[art]['minv']) ,max=float(self.param[art]['maxv']), vary=self.param[art]['vary']);
				
		
		#Find the index of the nearest point in the scale.
		
		data = np.array(self.autoNorm).astype(np.float64).reshape(-1)
		scale = np.array(self.autotime).astype(np.float64).reshape(-1)
		self.indx_L = int(np.argmin(np.abs(scale -  self.parentFn.dr.xpos)))
		self.indx_R = int(np.argmin(np.abs(scale -  self.parentFn.dr1.xpos)))
		
		#Run the fitting.
		res = minimize(self.residual, param, args=(scale[self.indx_L:self.indx_R+1],data[self.indx_L:self.indx_R+1], self.parentFn.def_options))

		#Repopulate the parameter object.
		for art in self.param:
			if self.param[art]['to_show'] == True and self.param[art]['calc'] == False:
				self.param[art]['value'] = param[art].value
				self.param[art]['stderr'] = float(param[art].stderr)
				
		#Extra parameters, which are not fit or inherited.
		#self.param['N_FCS']['value'] = np.round(1/self.param['GN0']['value'],4)


		self.residualVar = res.residual
		output = fit_report(param)
		print 'residual',res.chisqr
		if(res.chisqr>0.05):
			print 'CAUTION DATA DID NOT FIT WELL CHI^2 >0.05',res.chisqr
			self.goodFit = False
		else:
			self.goodFit = True
		self.fitted = True
		self.chisqr = res.chisqr
		
		self.localTime =  time.asctime( time.localtime(time.time()) )
		
		
		
		#self.parentFn.updateTableFirst();
		self.model_autoNorm = equation_(param, scale[self.indx_L:self.indx_R+1],self.parentFn.def_options)
		self.model_autotime = scale[self.indx_L:self.indx_R+1]
		#self.parentFn.on_show()

		#self.parentFn.axes.plot(model_autotime,model_autoNorm, 'o-')
		#self.parentFn.canvas.draw();
	
	def load_from_file(self,channel):
		tscale = [];
		tdata = [];
		int_tscale =[];
		int_tdata=[];

		
		if self.ext == 'fcs':
			


			corrObj = self
			text =[0]
			r_obj = csv.reader(open(corrObj.filepath, 'rb'),delimiter='\t')
			title = r_obj.next()
			line = r_obj.next()
			line = r_obj.next()
			name = line[1].split(' = ')[1]
			read = True
			while  read == True:
				
				corrObj = corrObject(self.filepath,self.parentFn);
				self.parentFn.objIdArr.append(corrObj)

				corrObj.name = name

				
				

				line = r_obj.next()
				text =[]
				for part in line:
					if part != '':
							text.append(part)
				

				#Reads to first correlation array text.
				while  text[0].split(' = ')[0] != 'CorrelationArray' or text[0].split(' = ')[1] == int(0):
					
					line = r_obj.next()
					text =[]
					for part in line:
						if part != '':
							text.append(part)
				line = r_obj.next()
				tdata = []
				tscale = []
				

				while  text[0].split(' = ')[0] != 'PhotonCountHistogramArraySize':
					
					try:
						line = r_obj.next()
					except:
						read = False
						break;
					text =[]
					for part in line:
						if part != '':
							text.append(part)
					if text.__len__() >1:
						tscale.append(float(text[0]))
						tdata.append(float(text[1]))

				if tdata.__len__() == 0:
					corrObj = []
					self.parentFn.objIdArr.pop(-1)
					break;

				channel = 0
				corrObj.siblings = None
				corrObj.autoNorm= np.array(tdata).astype(np.float64).reshape(-1)
				corrObj.autotime= np.array(tscale).astype(np.float64).reshape(-1)*1000
				corrObj.name = corrObj.name+'-CH'+str(channel)
				corrObj.ch_type = channel;
				corrObj.param = copy.deepcopy(self.parentFn.def_param)
				self.parentFn.fill_series_list()
				
				






		if self.ext == 'SIN':
			self.parentFn.objIdArr.append(self.objId)
			proceed = False
			
			for line in csv.reader(open(self.filepath, 'rb'),delimiter='\t'):
				
				if proceed =='correlated':
					if line ==[]:
						proceed =False;
					else:
						tscale.append(float(line[0]))
						tdata.append(float(line[channel+1]))
				if proceed =='intensity':
					
					if line ==[]:
						proceed=False;
					elif line.__len__()> 1:
						
						int_tscale.append(float(line[0]))
						int_tdata.append(float(line[channel+1]))
				if (str(line)  == "[\'[CorrelationFunction]\']"):
					proceed = 'correlated';
				elif (str(line)  == "[\'[IntensityHistory]\']"):
					proceed = 'intensity';
				
				
				
			
			self.siblings = None
			self.autoNorm= np.array(tdata).astype(np.float64).reshape(-1)
			self.autotime= np.array(tscale).astype(np.float64).reshape(-1)*1000
			self.name = self.name+'-CH'+str(channel)
			self.ch_type = channel;
			#self.prepare_for_fit()

			
			
			


			#Average counts per bin. For it to be seconds (Hz), must divide by duration.
			unit = int_tscale[-1]/(int_tscale.__len__()-1)
			#And to be in kHz we divide by 1000.
			self.kcount = np.average(np.array(int_tdata)/unit)/1000
			self.param = copy.deepcopy(self.parentFn.def_param)
			self.parentFn.fill_series_list()
			
		
					#Where we add the names.

		
		if self.ext == 'csv' or self.ext =='CSV':
			r_obj = csv.reader(open(self.filepath, 'rb'))
			line_one = r_obj.next()
			if line_one.__len__()>1:
				
					if float(line_one[1]) == 2:
						
						version = 2
					else:
						print 'version not known:',line_one[1]
					
				
			else:
				version = 1

			if version == 1:
				self.parentFn.objIdArr.append(self)
				
				c = 0
				
				for line in csv.reader(open(self.filepath, 'rb')):
					if (c >0):
						tscale.append(line[0])
						tdata.append(line[1])
					c +=1;

				self.autoNorm= np.array(tdata).astype(np.float64).reshape(-1)
				self.autotime= np.array(tscale).astype(np.float64).reshape(-1)
				self.name = self.name+'-CH'+str(0)
				self.ch_type = 0
				self.datalen= len(tdata)

				self.param = copy.deepcopy(self.parentFn.def_param)
				self.parentFn.fill_series_list()
			if version >= 2:
				
					
				numOfCH = float(r_obj.next()[1])
				
				if numOfCH == 1:
					self.parentFn.objIdArr.append(self)
					self.type =str(r_obj.next()[1])
					self.ch_type = int(r_obj.next()[1])
					self.name = self.name+'-CH'+str(self.ch_type)
					
					line = r_obj.next()

					while  line[0] != 'Time (ns)':
						if line[0] == 'kcount':
							self.kcount = float(line[1])
						if line[0] == 'numberNandB':
							self.numberNandB = float(line[1])
						if line[0] == 'brightnessNandB':
							self.brightnessNandB =  float(line[1])
						if line[0] == 'CV':
							self.CV =  float(line[1])
						if line[0] == 'carpet pos':
							carpet = int(line[1])
						if line[0] == 'pc':
							pc_text = int(line[1])
						if line[0] == 'pbc_f0':
							self.pbc_f0 = float(line[1])
						if line[0] == 'pbc_tb':
							self.pbc_tb = float(line[1])
						
						line = r_obj.next()

					
					if pc_text != False:
						self.name = self.name +'_pc_m'+str(pc_text)
						
					
					tscale = []
					tdata = []
					null = r_obj.next()
					line = r_obj.next()
					while  line[0] != 'end':

						tscale.append(line[0])
						tdata.append(line[1])
						line = r_obj.next()

					self.autoNorm= np.array(tdata).astype(np.float64).reshape(-1)
					self.autotime= np.array(tscale).astype(np.float64).reshape(-1)
					self.siblings = None
					self.param = copy.deepcopy(self.parentFn.def_param)
					self.parentFn.fill_series_list()

				if numOfCH == 2:
					corrObj2 = corrObject(self.filepath,self.parentFn);
					corrObj3 = corrObject(self.filepath,self.parentFn);

					self.parentFn.objIdArr.append(self)
					self.parentFn.objIdArr.append(corrObj2)
					self.parentFn.objIdArr.append(corrObj3)
					
					line_type = r_obj.next()
					self.type = str(line_type[1])
					corrObj2.type = str(line_type[1])
					corrObj3.type = str(line_type[1])
					

					line_ch = r_obj.next()
					self.ch_type = int(line_ch[1])
					corrObj2.ch_type = int(line_ch[2])
					corrObj3.ch_type = int(line_ch[3])
					
					self.name = self.name+'-CH'+str(self.ch_type)
					corrObj2.name = corrObj2.name+'-CH'+str(corrObj2.ch_type)
					corrObj3.name = corrObj3.name+'-CH'+str(corrObj3.ch_type)

					line = r_obj.next()
					while  line[0] != 'Time (ns)':
						if line[0] == 'kcount':
							self.kcount = float(line[1])
							corrObj2.kcount = float(line[2])
						if line[0] == 'numberNandB':
							self.numberNandB = float(line[1])
							corrObj2.numberNandB =  float(line[2])
						if line[0] == 'brightnessNandB':
							self.brightnessNandB =  float(line[1])
							corrObj2.brightnessNandB =  float(line[2])
						if line[0] == 'CV':
							self.CV =  float(line[1])
							corrObj2.CV = float(line[2])
							corrObj3.CV = float(line[3])
						if line[0] == 'carpet pos':
							self.carpet_position = int(line[1])
						if line[0] == 'pc':
							pc_text = int(line[1])
						if line[0] == 'pbc_f0':
							self.pbc_f0 = float(line[1])
							corrObj2.pbc_f0 = float(line[2])
						if line[0] == 'pbc_tb':
							self.pbc_tb = float(line[1])
							corrObj2.pbc_tb = float(line[2])
						
						line = r_obj.next()
					
					
					
					if pc_text != False:
						self.name = self.name +'_pc_m'+str(pc_text)
						corrObj2.name = corrObj2.name +'_pc_m'+str(pc_text)
						corrObj3.name = corrObj3.name +'_pc_m'+str(pc_text)
					
					null = r_obj.next()
					line = r_obj.next()
					tscale = []
					tdata0 = []
					tdata1 = []
					tdata2 = []
					while  line[0] != 'end':

						tscale.append(line[0])
						tdata0.append(line[1])
						tdata1.append(line[2])
						tdata2.append(line[3])
						line = r_obj.next()

					
					self.autotime= np.array(tscale).astype(np.float64).reshape(-1)
					corrObj2.autotime= np.array(tscale).astype(np.float64).reshape(-1)
					corrObj3.autotime= np.array(tscale).astype(np.float64).reshape(-1)

					self.autoNorm= np.array(tdata0).astype(np.float64).reshape(-1)
					corrObj2.autoNorm= np.array(tdata1).astype(np.float64).reshape(-1)
					corrObj3.autoNorm= np.array(tdata2).astype(np.float64).reshape(-1)
					
					self.siblings = [corrObj2,corrObj3]
					corrObj2.siblings = [self,corrObj3]
					corrObj3.siblings = [self,corrObj2]
					
					self.param = copy.deepcopy(self.parentFn.def_param)
					corrObj2.param = copy.deepcopy(self.parentFn.def_param)
					corrObj3.param = copy.deepcopy(self.parentFn.def_param)
					self.parentFn.fill_series_list()


				








import numpy as np
import copy
"""FCS Bulk Correlation Software

    Copyright (C) 2015  Dominic Waithe

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
"""
def initialise_fcs(int_obj):
	#default options for the fitting.
		int_obj.def_options ={}
		
		int_obj.def_options['Diff_eq'] = 1
		int_obj.def_options['Diff_species'] = 1
		int_obj.def_options['Triplet_eq'] = 1
		int_obj.def_options['Triplet_species'] = 1

		int_obj.def_options['Dimen'] =1
		

		
		A1 = {'alias':'A1','value':1.0,'minv':0.0,'maxv':1.0,'vary':False,'to_show':True,'stdev':False,'calc':False}
		A2 = {'alias':'A2','value':1.0,'minv':0.0,'maxv':1.0,'vary':False,'to_show':True,'calc':False}
		A3 = {'alias':'A3','value':1.0,'minv':0.0,'maxv':1.0,'vary':False,'to_show':True,'calc':False}		
		#The offset
		offset = { 'alias':'offset','value':0.01,'minv':-1.0,'maxv':1.0,'vary':True,'to_show':True,'calc':False}
		#int_obj.defin.add('offset', value=0.0, min=-1.0,max=5.0,vary=False)
		#The amplitude
		GN0 = {'alias':'GN0','minv':0.001,'value':1,'maxv':1.0,'vary':True,'to_show':True,'calc':False}
		#int_obj.def_param.add('GN0', value=1.0, vary=True)
		#The alpha value
		#int_obj.def_param.add('alpha', value=1.0, min=0,max=1.0, vary=True)
		#lateral diffusion coefficent
		txy1 = {'alias':'txy1','value':0.01,'minv':0.001,'maxv':2000.0,'vary':True,'to_show':True,'calc':False}
		txy2 = {'alias':'txy2','value':0.01,'minv':0.001,'maxv':2000.0,'vary':True,'to_show':True,'calc':False}
		txy3 = {'alias':'txy3','value':0.01,'minv':0.001,'maxv':2000.0,'vary':True,'to_show':True,'calc':False}

		

		alpha1 = {'alias':'alpha1','value':1.0,'minv':0.0,'maxv':2.0,'vary':True,'to_show':True,'calc':False}
		alpha2 = {'alias':'alpha2','value':1.0,'minv':0.0,'maxv':2.0,'vary':True,'to_show':True,'calc':False}
		alpha3 = {'alias':'alpha3','value':1.0,'minv':0.0,'maxv':2.0,'vary':True,'to_show':True,'calc':False}
		
		tz1 = {'alias':'tz1','value':1.0,'minv':0.0,'maxv':1.0,'vary':True,'to_show':True,'calc':False}
		tz2 = {'alias':'tz2','value':1.0,'minv':0.0,'maxv':1.0,'vary':True,'to_show':True,'calc':False}
		tz3 = {'alias':'tz3','value':1.0,'minv':0.0,'maxv':1.0,'vary':True,'to_show':True,'calc':False}

		#Axial ratio coefficient
		
		AR1 = {'alias':'AR1','value':1.0,'minv':0.001,'maxv':1000.0,'vary':True,'to_show':True,'calc':False}
		AR2 = {'alias':'AR2','value':1.0,'minv':0.001,'maxv':1000.0,'vary':True,'to_show':True,'calc':False}
		AR3 = {'alias':'AR3','value':1.0,'minv':0.001,'maxv':1000.0,'vary':True,'to_show':True,'calc':False}

		B1 = {'alias':'B1','value':1.0,'minv':0.001,'maxv':1000.0,'vary':True,'to_show':True,'calc':False}
		B2 = {'alias':'B2','value':1.0,'minv':0.001,'maxv':1000.0,'vary':True,'to_show':True,'calc':False}
		B3 = {'alias':'B3','value':1.0,'minv':0.001,'maxv':1000.0,'vary':True,'to_show':True,'calc':False}

		T1 = {'alias':'T1','value':1.0,'minv':0.0,'maxv':1000.0,'vary':True,'to_show':True,'calc':False}
		T2 = {'alias':'T2','value':1.0,'minv':0.0,'maxv':1000.0,'vary':True,'to_show':True,'calc':False}
		T3 = {'alias':'T3','value':1.0,'minv':0.0,'maxv':1000.0,'vary':True,'to_show':True,'calc':False}
	
		tauT1 = {'alias':'tauT1','value':0.055,'minv':0.001,'maxv':1000.0,'vary':True,'to_show':True,'calc':False}
		tauT2 = {'alias':'tauT2','value':0.055,'minv':0.001,'maxv':1000.0,'vary':True,'to_show':True,'calc':False}
		tauT3 = {'alias':'tauT3','value':0.005,'minv':0.001,'maxv':1000.0,'vary':True,'to_show':True,'calc':False}

		N_FCS = {'alias':'N (FCS)','value':0.0,'minv':0.001,'maxv':1000.0,'vary':True,'to_show':True,'calc':True}
		cpm = {'alias':'cpm (kHz)','value':0.0,'minv':0.001,'maxv':1000.0,'vary':True,'to_show':True,'calc':True}
		N_mom = {'alias':'N (mom)','value':0.0,'minv':0.001,'maxv':1000.0,'vary':True,'to_show':True,'calc':True}
		bri = {'alias':'bri (kHz)','value':0.0,'minv':0.001,'maxv':1000.0,'vary':True,'to_show':True,'calc':True}
		CV = {'alias':'Coincidence','value':0.0,'minv':0.001,'maxv':1000.0,'vary':True,'to_show':True,'calc':True}
		f0 = {'alias':'PBC f0','value':0.0,'minv':0.001,'maxv':1000.0,'vary':True,'to_show':True,'calc':True}
		overtb = {'alias':'PBC tb','value':0.0,'minv':0.001,'maxv':1000.0,'vary':True,'to_show':True,'calc':True}

		
		ACAC = {'alias':'ACAC','value':0.0,'minv':0.001,'maxv':1000.0,'vary':True,'to_show':True,'calc':True}
		ACCC = {'alias':'ACCC','value':0.0,'minv':0.001,'maxv':1000.0,'vary':True,'to_show':True,'calc':True}


		int_obj.def_param ={'A1':A1,'A2':A2,'A3':A3,'txy1':txy1,'txy2':txy2,'txy3':txy3,'offset':offset,'GN0':GN0,'alpha1':alpha1,'alpha2':alpha2,'alpha3':alpha3,'tz1':tz1,'tz2':tz2,'tz3':tz3,'AR1':AR1,'AR2':AR2,'AR3':AR3,'B1':B1,'B2':B2,'B3':B3,'T1':T1,'T2':T2,'T3':T3,'tauT1':tauT1,'tauT2':tauT2,'tauT3':tauT3}
		int_obj.def_param['N_FCS'] = N_FCS
		int_obj.def_param['cpm'] = cpm
		int_obj.def_param['N_mom'] = N_mom
		int_obj.def_param['bri'] = bri
		int_obj.def_param['CV'] = CV
		int_obj.def_param['f0'] = f0
		int_obj.def_param['overtb'] = overtb

		int_obj.def_param['ACAC'] = ACAC
		int_obj.def_param['ACCC'] = ACCC

		
		int_obj.order_list = ['offset','GN0','N_FCS','cpm','A1','A2','A3','txy1','txy2','txy3','tz1','tz2','tz3','alpha1','alpha2','alpha3','AR1','AR2','AR3','B1','B2','B3','T1','T2','T3','tauT1','tauT2','tauT3','N_mom','bri','CV','f0','overtb','ACAC','ACCC']


def decide_which_to_show(int_obj):
	
	for art in int_obj.objId_sel.param:
		if int_obj.objId_sel.param[art]['to_show'] == True:
			int_obj.objId_sel.param[art]['to_show'] = False

		int_obj.objId_sel.param[ 'offset']['to_show'] = True
		int_obj.objId_sel.param[ 'GN0']['to_show'] = True

		int_obj.def_options['Diff_species'] = int_obj.diffNumSpecSpin.value()
		int_obj.def_options['Triplet_species'] =int_obj.tripNumSpecSpin.value()
		
		#Optional parameters
		for i in range(1,int_obj.def_options['Diff_species']+1):
			int_obj.objId_sel.param['A'+str(i)]['to_show'] = True
			int_obj.objId_sel.param['txy'+str(i)]['to_show'] = True
			int_obj.objId_sel.param['alpha'+str(i)]['to_show'] = True
			#2 in this case corresponds to 3D:
			if int_obj.def_options['Dimen'] == 2:
				if int_obj.def_options['Diff_eq'] == 1:
					int_obj.objId_sel.param['tz'+str(i)]['to_show'] = True
					
				if int_obj.def_options['Diff_eq'] == 2:
					int_obj.objId_sel.param[ 'AR'+str(i)]['to_show'] = True
						
		if int_obj.def_options['Triplet_eq'] == 2:
				#Triplet State equation1
				for i in range(1,int_obj.tripNumSpecSpin.value()+1):			 
					int_obj.objId_sel.param['B'+str(i)]['to_show'] = True
					int_obj.objId_sel.param['tauT'+str(i)]['to_show'] = True
					
		if int_obj.def_options['Triplet_eq'] == 3:
				#Triplet State equation2
				for i in range(1,int_obj.tripNumSpecSpin.value()+1):
					int_obj.objId_sel.param['T'+str(i)]['to_show'] = True
					int_obj.objId_sel.param['tauT'+str(i)]['to_show'] = True
		calc_param_fcs(int_obj,objId=int_obj.objId_sel)

def update_each(int_obj,text):
		"""Will try and populate paramaters with what is present in the inteface, but if new option will goto the default"""
		try:
			exec("valueV = int_obj."+text+"_value.value()"); exec("minV = int_obj."+text+"_min.value()"); exec("maxV = int_obj."+text+"_max.value()"); exec("varyV = int_obj."+text+"_vary.isChecked()");
			
			int_obj.objId_sel.param[text]['value'] = valueV 
			int_obj.objId_sel.param[text]['minv'] = minV
			int_obj.objId_sel.param[text]['maxv'] = maxV
			int_obj.objId_sel.param[text]['vary'] = varyV
		except:
			
			int_obj.objId_sel.param[text] = copy.deepcopy(int_obj.def_param[text])
def update_param_fcs(int_obj):
		"""Depending on the menu options this function will update the params of the current data set. """
		if int_obj.objId_sel ==None:
			return
		decide_which_to_show(int_obj)
		#Set all the parameters to not show.

		for art in int_obj.objId_sel.param:
			if int_obj.objId_sel.param[art]['to_show'] == True:
				update_each(int_obj, art)
		
		calc_param_fcs(int_obj,objId=int_obj.objId_sel)
def calc_param_fcs(int_obj,objId):
		#Calculated parameters.
		try:
			objId.param['N_FCS']['value'] = 1/objId.param['GN0']['value']
			objId.param['N_FCS']['to_show'] = True
		except:
			objId.param['N_FCS']['value'] = 1
			objId.param['N_FCS']['to_show'] = False
		
		
		try:
			objId.param['cpm']['value'] = float(objId.kcount)/(1/objId.param['GN0']['value'])
			objId.param['cpm']['to_show'] = True
		except:
			objId.param['cpm']['value'] = 1
			objId.param['cpm']['to_show'] = False
		try:
			objId.param['N_mom']['value'] =  float(objId.numberNandB)
			objId.param['N_mom']['to_show'] = True
		except:
			objId.param['N_mom']['value'] =  1
			objId.param['N_mom']['to_show'] = False
		try:
			objId.param['bri']['value'] = float(objId.brightnessNandB)
			objId.param['bri']['to_show'] = True
		except:
			objId.param['bri']['value'] = 1
			objId.param['bri']['to_show'] = False
		try:
			objId.param['CV']['value'] = float(objId.CV)
			objId.param['CV']['to_show'] = True
		except:
			pass
		try:
			objId.param['f0']['value'] = float(objId.pbc_f0)
			objId.param['f0']['to_show'] = True
			objId.param['overtb']['value'] = float(objId.pbc_tb)
			objId.param['overtb']['to_show'] = True
		except:
			pass

		if int_obj.objIdArr != [] and objId.siblings !=None and objId.ch_type != 2:
					
					if objId.siblings[0].fitted == True:
						
						objId.param['ACAC']['value'] = float(objId.param['GN0']['value'])/float(objId.siblings[0].param['GN0']['value'])
						objId.param['ACAC']['to_show'] = True
						objId.param['ACCC']['value'] = float(objId.param['GN0']['value'])/float(objId.siblings[1].param['GN0']['value'])
						objId.param['ACCC']['to_show'] = True
				

def equation_(param, tc,options):
	"""This is equation for fitting"""

	#A1 is relative component of fluorescent species
	#tc is tau.
	#txy1 is xy difusion   for fluorescent species one.
	#alpha1 is
	#tz1 is z diffusion for fluorescent species one.
	offset =param['offset'].value; 
	GN0 =param['GN0'].value; 
	
	
	if(options['Dimen'] == 2):
		if(options['Diff_eq']==1):
			#Equation 1A with 3D term.
			if (options['Diff_species'] == 1):
				A1 = param['A1'].value; txy1 = param['txy1'].value; alpha1 = param['alpha1'].value;tz1 = param['tz1'].value;
				#For one diffusing species
				GDiff = (A1*(((1+((tc/txy1)**alpha1))**-1)*((1+(tc/tz1))**-0.5)))
			elif (options['Diff_species'] == 2):
				A1 = param['A1'].value; txy1 = param['txy1'].value; alpha1 = param['alpha1'].value;tz1 = param['tz1'].value;
				A2 = param['A2'].value; txy2 = param['txy2'].value; alpha2 = param['alpha2'].value;tz2 = param['tz2'].value;
				param['A2'].value = 1.0-A1
				A2 = param['A2'].value
				#For two diffusing species
				GDiff = (A1*(((1+((tc/txy1)**alpha1))**-1)*((1+(tc/tz1))**-0.5)))+ (A2*(((1+((tc/txy2)**alpha2))**-1)*((1+(tc/tz2))**-0.5)))
			elif (options['Diff_species'] == 3):
				A1 = param['A1'].value; txy1 = param['txy1'].value; alpha1 = param['alpha1'].value;tz1 = param['tz1'].value;
				A2 = param['A2'].value; txy2 = param['txy2'].value; alpha2 = param['alpha2'].value;tz2 = param['tz2'].value;
				A3 = param['A3'].value; txy3 = param['txy3'].value; alpha3 = param['alpha3'].value;tz3 = param['tz3'].value;
				param['A2'].value = 1.0-A1-A3
				A2 = param['A2'].value
				param['A3'].value = 1.0-A2-A1
				A3 = param['A3'].value
				#For three diffusing species
				GDiff = (A1*(((1+((tc/txy1)**alpha1))**-1)*((1+(tc/tz1))**-0.5)))+ (A2*(((1+((tc/txy2)**alpha2))**-1)*((1+(tc/tz2))**-0.5)))+ (A3*(((1+((tc/txy3)**alpha3))**-1)*((1+(tc/tz3))**-0.5)))
		elif(options['Diff_eq']==2):
			if (options['Diff_species'] == 1):
				A1 = param['A1'].value; txy1 = param['txy1'].value; alpha1 = param['alpha1'].value;AR1 = param['AR1'].value;
				#For one diffusing species
				GDiff = (A1*(((1+((tc/txy1)**alpha1))**-1)*(((1+(tc/((AR1**2)*txy1)))**-0.5))))
			elif (options['Diff_species'] == 2):
				A1 = param['A1'].value; txy1 = param['txy1'].value; alpha1 = param['alpha1'].value;AR1 = param['AR1'].value;
				A2 = param['A2'].value; txy2 = param['txy2'].value; alpha2 = param['alpha2'].value;AR2 = param['AR2'].value;
				param['A2'].value = 1.0-A1
				A2 = param['A2'].value
				#For two diffusing species
				GDiff = (A1*(((1+((tc/txy1)**alpha1))**-1)*(((1+(tc/((AR1**2)*txy1)))**-0.5))))+(A2*(((1+((tc/txy2)**alpha2))**-1)*(((1+(tc/((AR2**2)*txy2)))**-0.5))))
			elif (options['Diff_species'] == 3):
				A1 = param['A1'].value; txy1 = param['txy1'].value; alpha1 = param['alpha1'].value;AR1 = param['AR1'].value;
				A2 = param['A2'].value; txy2 = param['txy2'].value; alpha2 = param['alpha2'].value;AR2 = param['AR2'].value;
				A3 = param['A3'].value; txy3 = param['txy3'].value; alpha3 = param['alpha3'].value;AR3 = param['AR3'].value;
				#For two diffusing species
				param['A2'].value = 1.0-A1-A3
				A2 = param['A2'].value
				param['A3'].value = 1.0-A2-A1
				A3 = param['A3'].value
				#For three diffusing species
				GDiff = (A1*(((1+((tc/txy1)**alpha1))**-1)*(((1+(tc/((AR1**2)*txy1)))**-0.5))))+(A2*(((1+((tc/txy2)**alpha2))**-1)*(((1+(tc/((AR2**2)*txy2)))**-0.5))))+(A3*(((1+((tc/txy3)**alpha3))**-1)*(((1+(tc/((AR3**2)*txy3)))**-0.5))))
																														   
	if(options['Dimen'] == 1):
		#Equation 1A with 2D term.
		if (options['Diff_species'] == 1):
			A1 = param['A1'].value; txy1 = param['txy1'].value; alpha1 = param['alpha1'].value;
			#For one diffusing species
			GDiff = (A1*(((1+((tc/txy1)**alpha1))**-1)))
		elif (options['Diff_species'] == 2):
			A1 = param['A1'].value; txy1 = param['txy1'].value; alpha1 = param['alpha1'].value;
			A2 = param['A2'].value; txy2 = param['txy2'].value; alpha2 = param['alpha2'].value;
			#For two diffusing species

			param['A2'].value = 1.0-A1
			A2 = param['A2'].value

			GDiff = (A1*(((1+(tc/txy1)**alpha1)**-1)))+(A2*(((1+(tc/txy2)**alpha2)**-1)))

			#if A1 +A2 != 1.0:
			#    GDiff = 99999999999
		elif (options['Diff_species'] == 3):
			A1 = param['A1'].value; txy1 = param['txy1'].value; alpha1 = param['alpha1'].value;
			A2 = param['A2'].value; txy2 = param['txy2'].value; alpha2 = param['alpha2'].value;
			A3 = param['A3'].value; txy3 = param['txy3'].value; alpha3 = param['alpha3'].value;
			#For two diffusing species
			param['A2'].value = 1.0-A1-A3
			A2 = param['A2'].value
			param['A3'].value = 1.0-A2-A1
			A3 = param['A3'].value
			#For three diffusing species
			GDiff = (A1*(((1+(tc/txy1)**alpha1)**-1)))+(A2*(((1+(tc/txy2)**alpha2)**-1)))+(A3*(((1+(tc/txy3)**alpha3)**-1)))
	
	if(options['Triplet_eq'] ==1):
		#For no triplets.
		GT = 1
	elif(options['Triplet_eq'] ==2):
		#Equation (2) 1st equation.
		if (options['Triplet_species'] == 1):
			B1 = param['B1'].value;tauT1 = param['tauT1'].value;
			#For one dark state.
			GT = 1 + (B1*np.exp(-tc/tauT1))
		elif (options['Triplet_species'] == 2):
			B1 = param['B1'].value;tauT1 = param['tauT1'].value;
			B2 = param['B2'].value;tauT2 = param['tauT2'].value;
			#For two dark state
			GT = 1 + (B1*np.exp(-tc/tauT1))+(B2*np.exp(-tc/tauT2))
		elif (options['Triplet_species'] == 3):
			B1 = param['B1'].value;tauT1 = param['tauT1'].value;
			B2 = param['B2'].value;tauT2 = param['tauT2'].value;
			B3 = param['B3'].value;tauT3 = param['tauT3'].value;
			#For three dark state
			GT = 1 + (B1*np.exp(-tc/tauT1))+(B2*np.exp(-tc/tauT2))+(B3*np.exp(-tc/tauT3))
	
	elif(options['Triplet_eq'] ==3):       
		#Equation (2) 2nd equation.
		if (options['Triplet_species'] == 1):
			T1 = param['T1'].value;tauT1 = param['tauT1'].value;
			#For one dark state.
			GT = 1- T1 + (T1*np.exp(-tc/tauT1))
		elif (options['Triplet_species'] == 2):
			T1 = param['T1'].value;tauT1 = param['tauT1'].value;
			T1 = param['T2'].value;tauT1 = param['tauT2'].value;
			#For two dark state.
			GT = 1- (T1+T2 )+ ((T1*np.exp(-tc/tauT1))+(T2*np.exp(-tc/tauT2)))
		elif (options['Triplet_species'] == 3):
			T1 = param['T1'].value;tauT1 = param['tauT1'].value;
			T2 = param['T2'].value;tauT1 = param['tauT2'].value;
			T3 = param['T3'].value;tauT1 = param['tauT3'].value;
			#For three dark state.
			GT = 1- (T1+T2+T3)+ ((T1*np.exp(-tc/tauT1))+(T2*np.exp(-tc/tauT2))+(T3*np.exp(-tc/tauT3)))
			
	return offset + (GN0*GDiff*GT)


# -*- coding: utf-8 -*-
"""
methods to open ALV .ASC files
"""
from __future__ import division

import os
import csv
import numpy as np


def openASC(dirname, filename):
    """ 
    Read data from a ALV .ASC files.
    """
    path = os.path.join(dirname, filename)
    with open(path, 'r') as openfile:
        Alldata = openfile.readlines()
    
    # Open special format?
    filetype = Alldata[0].strip() 
    if filetype in ["ALV-7004/USB"]:
        return openASC_ALV_7004_USB(path)
    else:
        # last resort
        return openASC_old(path)


def openASC_old(path):
    """ Read data from a .ASC file, created by
        some ALV-6000 correlator.

            ALV-6000/E-WIN Data
            Date :    "2/20/2012"
            ...
            "Correlation"
              1.25000E-004      3.00195E-001
              2.50000E-004      1.13065E-001
              3.75000E-004      7.60367E-002
              5.00000E-004      6.29926E-002
              6.25000E-004      5.34678E-002
              7.50000E-004      4.11506E-002
              8.75000E-004      4.36752E-002
              1.00000E-003      4.63146E-002
              1.12500E-003      3.78226E-002
            ...
              3.35544E+004     -2.05799E-006
              3.77487E+004      4.09032E-006
              4.19430E+004      4.26295E-006
              4.61373E+004      1.40265E-005
              5.03316E+004      1.61766E-005
              5.45259E+004      2.19541E-005
              5.87202E+004      3.26527E-005
              6.29145E+004      2.72920E-005

            "Count Rate"
               1.17188          26.77194
               2.34375          26.85045
               3.51563          27.06382
               4.68750          26.97932
               5.85938          26.73694
               7.03125          27.11332
               8.20313          26.81376
               9.37500          26.82741
              10.54688          26.88801
              11.71875          27.09710
              12.89063          27.13209
              14.06250          27.02200
              15.23438          26.95287
              16.40625          26.75657
              17.57813          26.43056
            ...
             294.14063          27.22597
             295.31250          26.40581
             296.48438          26.33497
             297.65625          25.96457
             298.82813          26.71902

        1. We are interested in the "Correlation" section,
        where the first column denotes tau in ms and the second row the
        correlation signal. Values are separated by a tabulator "\t" (some " ").

        2. We are also interested in the "Count Rate" section. Here the times
        are saved as seconds and not ms like above.

        3. There is some kind of mode where the ALV exports five runs at a
        time and averages them. The sole correlation data is stored in the
        file, but the trace is only stored as average or something.
        So I would not recommend this. However, I added support for this.
        PyCorrFit then only imports the average data.
         ~ Paul, 2012-02-20
        Correlation data starts at "Correlation (Multi, Averaged)".

        Returns:
        [0]:
         An array with tuples containing two elements:
         1st: tau in ms
         2nd: corresponding correlation signal
        [1]:
         Intensity trace:
         1st: time in ms
         2nd: Trace in kHz
        [2]:
         An array with N elements, indicating, how many curves we are opening
         from the file. Elements can be names and must be convertible to
         strings.
    """
    filename = os.path.basename(path)
    with open(path, 'r') as openfile:
        Alldata = openfile.readlines()
    # End of trace
    EndT = Alldata.__len__()
    ## Correlation function
    # Find out where the correlation function is
    for i in np.arange(len(Alldata)):
        if Alldata[i].startswith('Mode'):
            mode = Alldata[i][5:].strip(' ":').strip().strip('"')
            single_strings = ["a-ch0", "a-ch1", "auto ch0", "auto ch1",
                              "fast auto ch0", "fast auto ch1",
                               ]
            if (mode.lower().count('single') or
                mode.lower().strip() in single_strings):
                single = True
                channel = mode.split(" ")[-1]
            else:
                # dual
                single = False
            # accc ?
            if mode.lower().count("cross") == 1:
                accc = "CC"
            else:
                accc = "AC"
        if Alldata[i].startswith('"Correlation'):
            # This tells us if there is only one curve or if there are
            # multiple curves with an average.
            if (Alldata[i].strip().lower() == 
                '"correlation (multi, averaged)"' ):
                multidata = True
            else:
                multidata = False
        if Alldata[i].startswith('"Correlation"'):
            # Start of correlation function
            StartC = i+1
        if Alldata[i].startswith('"Correlation (Multi, Averaged)"'):
            # Start of AVERAGED correlation function !!!
            # There are several curves now.
            StartC = i+2
        if Alldata[i].replace(" ", "").lower().strip() == '"countrate"':
            # takes care of "Count Rate" and "Countrate"
            # End of correlation function
            EndC = i-1
            # Start of trace (goes until end of file)
            StartT = i+1
        if Alldata[i].startswith('Monitor Diode'):
            EndT = i-1
    # Get the header
    Namedata = Alldata.__getslice__(StartC-1, StartC)
    ## Define *curvelist*
    curvelist = csv.reader(Namedata, delimiter='\t').next()
    if len(curvelist) <= 2:
        # Then we have just one single correlation curve
        curvelist = [""]
    else:
        # We have a number of correlation curves. We need to specify
        # names for them. We take these names from the headings.
        # Lag times not in the list
        curvelist.remove(curvelist[0])
        # Last column is empty
        curvelist.remove(curvelist[-1])
    ## Correlation function
    Truedata = Alldata.__getslice__(StartC, EndC)
    readdata = csv.reader(Truedata, delimiter='\t')
    # Add lists to *data* according to the length of *curvelist*
    data = [[]]*len(curvelist)
    # Work through the rows in the read data
    for row in readdata:
        for i in np.arange(len(curvelist)):
            if len(row) > 0:
                data[i].append( (np.float(row[0]), np.float(row[i+1])) )
    ## Trace
    # Trace is stored in two columns
    # 1st column: time [s]
    # 2nd column: trace [kHz] 
    # Get the trace
    Tracedata = Alldata.__getslice__(StartT, EndT)
    timefactor = 1000 # because we want ms instead of s
    readtrace = csv.reader(Tracedata, delimiter='\t')
    trace = list()
    trace2 = list()
    # Work through the rows
    for row in readtrace:
        # time in ms, countrate
        trace.append(list())
        trace[0].append((np.float(row[0])*timefactor,
                         np.float(row[1])))
        # Only trace[0] contains the trace!
        for i in np.arange(len(curvelist)-1):
            trace.append(list())
            trace[i+1].append((np.float(row[0])*timefactor, 0))
        if not single:
            k = len(curvelist)/2
            if int(k) != k:
                print "Problem with ALV data. Single mode not recognized."
            # presumably dual mode. There is a second trace
            # time in ms, countrate
            trace2.append(list())
            trace2[0].append((np.float(row[0])*timefactor,
                              np.float(row[2])))
            # Only trace2[0] contains the trace!
            for i in np.arange(len(curvelist)-1):
                trace2.append(list())
                trace2[i+1].append((np.float(row[0])*timefactor, 0))

    # group the resulting curves
    corrlist = list()
    tracelist = list()
    typelist = list()
        
    if single:
        # We only have several runs and one average
        # split the trace into len(curvelist)-1 equal parts
        if multidata:
            nav = 1
        else:
            nav = 0
        splittrace = mysplit(trace[0], len(curvelist)-nav)
        i = 0
        for t in range(len(curvelist)):
            typ = curvelist[t]
            if typ.lower()[:7] == "average":
                typelist.append("{} average".format(channel))
                corrlist.append(np.array(data[t]))
                tracelist.append(np.array(trace[0]))
            else:
                typelist.append("{} {}".format(accc, channel))
                corrlist.append(np.array(data[t]))
                tracelist.append(splittrace[i])
                i += 1
    elif accc == "AC":
        # Dual mode, autocorrelation
        # We now have two averages and two different channels.
        # We now have two traces.
        # The data is assembled in blocks. That means the first block
        # contains an average and the data of channel 0 and the second
        # block contains data and average of channel 1. We can thus
        # handle the data from 0 to len(curvelist)/2 and from
        # len(curvelist)/2 to len(curvelist) as two separate data sets.
        # CHANNEL 0
        if multidata:
            nav = 1
        else:
            nav = 0
        channel = "CH0"
        splittrace = mysplit(trace[0], len(curvelist)/2-nav)
        i = 0
        for t in range(int(len(curvelist)/2)):
            typ = curvelist[t]
            if typ.lower()[:7] == "average":
                typelist.append("{} average".format(channel))
                corrlist.append(np.array(data[t]))
                tracelist.append(np.array(trace[0]))
            else:
                typelist.append("{} {}".format(accc, channel))
                corrlist.append(np.array(data[t]))
                tracelist.append(splittrace[i])
                i += 1
        # CHANNEL 1
        channel = "CH1"
        splittrace2 = mysplit(trace2[0], len(curvelist)/2-nav)
        i = 0
        for t in range(int(len(curvelist)/2),int(len(curvelist))):
            typ = curvelist[t]
            if typ.lower()[:7] == "average":
                typelist.append("{} average".format(channel))
                corrlist.append(np.array(data[t]))
                tracelist.append(np.array(trace2[0]))
            else:
                typelist.append("{} {}".format(accc, channel))
                corrlist.append(np.array(data[t]))
                tracelist.append(splittrace2[i])
                i += 1
    elif accc == "CC":
        if multidata:
            nav = 1
        else:
            nav = 0
        # Dual mode, cross-correlation
        channel = "CC01"
        splittrace = mysplit(trace[0], len(curvelist)/2-nav)
        splittrace2 = mysplit(trace2[0], len(curvelist)/2-nav)
        i = 0
        for t in range(int(len(curvelist)/2)):
            typ = curvelist[t]
            if typ.lower()[:7] == "average":
                typelist.append("{} average".format(channel))
                corrlist.append(np.array(data[t]))
                tracelist.append([np.array(trace[0]),
                                  np.array(trace2[0])  ])
            else:
                typelist.append("{} {}".format(accc, channel))
                corrlist.append(np.array(data[t]))
                tracelist.append([splittrace[i], splittrace2[i]])
                i += 1
        # CHANNEL 1
        channel = "CC10"
        i = 0
        for t in range(int(len(curvelist)/2),int(len(curvelist))):
            typ = curvelist[t]
            if typ.lower()[:7] == "average":
                typelist.append("{} average".format(channel))
                corrlist.append(np.array(data[t]))
                # order must be the same as above
                tracelist.append([np.array(trace[0]),
                                  np.array(trace2[0])  ])
            else:
                typelist.append("{} {}".format(accc, channel))
                corrlist.append(np.array(data[t]))
                # order must be the same as above
                tracelist.append([splittrace[i], splittrace2[i]])
                i += 1
    else:
        print "Could not detect data file format for: {}".format(filename)
        corrlist = np.array(data)
        tracelist = np.array(trace)
        typelist = curvelist

    dictionary = dict()
    dictionary["Correlation"] = corrlist
    dictionary["Trace"] = tracelist
    dictionary["Type"] = typelist
    filelist = list()
    for i in curvelist:
        filelist.append(filename)
    dictionary["Filename"] = filelist
    
    return dictionary


def openASC_ALV_7004_USB(path):
    """
    Opens ALV file format with header information "ALV-7004/USB" 
    
    This is a single-run file format.
    - data is identified by 4*"\t"
    - count rate is identified by string (also "countrate")
    - allzero-correlations are removed

    "Correlation"
      2.50000E-005     -9.45478E-001     -1.00000E+000      5.22761E-002      3.05477E-002
      5.00000E-005      6.73734E-001     -2.59938E-001      3.17894E-002      4.24466E-002
      7.50000E-005      5.30716E-001      3.21605E-001      5.91051E-002      2.93061E-002
      1.00000E-004      3.33292E-001      1.97860E-001      3.24102E-002      3.32379E-002
      1.25000E-004      2.42538E-001      1.19988E-001      4.37917E-002      3.05477E-002
      1.50000E-004      1.86396E-001      1.23318E-001      5.66218E-002      2.25806E-002
      1.75000E-004      1.73836E-001      8.53991E-002      4.64819E-002      3.46865E-002
      2.00000E-004      1.48080E-001      9.35377E-002      4.37917E-002      4.17223E-002
    [...]
      1.00663E+004      2.80967E-005     -2.23975E-005     -7.08272E-005      5.70470E-005
      1.09052E+004      9.40185E-005      2.76261E-004      1.29745E-004      2.39958E-004
      1.17441E+004     -2.82103E-004     -1.97386E-004     -2.88753E-004     -2.60987E-004
      1.25829E+004      1.42069E-004      3.82018E-004      6.03932E-005      5.40363E-004
    
    "Count Rate"
           0.11719         141.83165          81.54211         141.83165          81.54211
           0.23438         133.70215          77.90344         133.70215          77.90344
           0.35156         129.67148          74.58858         129.67148          74.58858
           0.46875         134.57133          79.53957         134.57133          79.53957
    [...]
          29.29688         143.78307          79.06236         143.78307          79.06236
          29.41406         154.80135          82.87147         154.80135          82.87147
          29.53125         187.43013          89.61197         187.43013          89.61197
          29.64844         137.82655          77.71597         137.82655          77.71597
    [...]


    """
    filename = os.path.basename(path)
    with open(path, 'r') as openfile:
        Alldata = openfile.readlines()
    
    # Find the different arrays
    # correlation array: "  "
    # trace array: "       "
    Allcorr = list()
    Alltrac = list()
    i=0
    intrace = False
    for item in Alldata:
        if item.lower().strip().strip('"').replace(" ", "") == "countrate":
            intrace = True
        i += 1
        if item.count("\t") == 4: 
            if intrace:
                it = item.split("\t")
                it = [ float(t.strip()) for t in it ]
                Alltrac.append(it)
            else:
                ic = item.split("\t")
                ic = [ float(c.strip()) for c in ic ]
                Allcorr.append(ic)
    Allcorr = np.array(Allcorr)
    Alltrac = np.array(Alltrac)
    
    # Allcorr: lag time, ac1, ac2, cc12, cc21
    # Alltrac: time, trace1, trace2, trace1, trace2
    assert np.allclose(Alltrac[:,1], Alltrac[:,3], rtol=.01), "unknown ALV file format"
    assert np.allclose(Alltrac[:,2], Alltrac[:,4], rtol=.01), "unknown ALV file format"
    
    guesstypelist = ["AC1", "AC2", "CC12", "CC21"]
    typelist = list()
    corrlist = list()
    tracelist = list()
    filelist = list()
    
    lagtime = Allcorr[:,0]
    time = Alltrac[:,0]*1000
    trace1 = np.dstack((time, Alltrac[:,1]))[0]
    trace2 = np.dstack((time, Alltrac[:,2]))[0]
    
    for i, typ in enumerate(guesstypelist):
        corr = np.dstack((lagtime, Allcorr[:,i+1]))[0]
        
        if not np.allclose(corr[:,1], np.zeros_like(lagtime)):
            # type
            typelist.append(typ)
            # correlation
            corrlist.append(corr)
            # trace
            if typ.count("CC"):
                tracelist.append([trace1, trace2])
            elif typ.count("AC1"):
                tracelist.append([trace1])
            elif typ.count("AC2"):
                tracelist.append([trace2])
            else:
                raise ValueError("Unknown ALV file format")
            # filename
            filelist.append(filename)

    dictionary = dict()
    dictionary["Correlation"] = corrlist
    dictionary["Trace"] = tracelist
    dictionary["Type"] = typelist
    dictionary["Filename"] = filelist
    return dictionary


def mysplit(a, n):
    """
       Split a trace into n equal parts by interpolation.
       The signal average is preserved, but the signal variance will
       decrease.
    """
    if n <= 1:
        return [np.array(a)]
    a = np.array(a)
    N = len(a)
    lensplit = np.int(np.ceil(N/n))

    # xp is actually rounded -> recalculate
    xp = np.linspace(a[:,0][0], a[:,0][-1], N,  endpoint=True)
    
    # let xp start at zero
    xp -= a[:,0][0]
    yp = a[:,1]
    
    # time frame for each new curve
    #dx = xp[-1]/n

    # perform interpolation of new trace
    x, newstep = np.linspace(0, xp[-1], lensplit*n,
                        endpoint=True, retstep=True)
    # interpolating reduces the variance and possibly changes the avg
    y = np.interp(x,xp,yp)
    
    data = np.zeros((lensplit*n,2))
    data[:,0] = x + newstep
    # make sure that the average stays the same:
    data[:,1] = y - np.average(y) + np.average(yp)
    return np.split(data,n)
    



# -*- coding: utf-8 -*-
"""
PyCorrFit

Module tools - average
Creates an average of curves.
"""


import numpy as np
import wx

from .. import misc
from .. import models as mdls

# Menu entry name
MENUINFO = ["&Average data", "Create an average curve from whole session."]

class Average(wx.Frame):
    # This tool is derived from a wx.frame.
    def __init__(self, parent):
        # Define a unique name that identifies this tool
        # Do not change this value. It is important for the Overlay tool
        # (selectcurves.py, *Wrapper_Tools*).
        self.MyName="AVERAGE"
        # parent is the main frame of PyCorrFit
        self.parent = parent
        # Get the window positioning correctly
        pos = self.parent.GetPosition()
        pos = (pos[0]+100, pos[1]+100)
        wx.Frame.__init__(self, parent=self.parent, title="Average curves",
            pos=pos, style=wx.DEFAULT_FRAME_STYLE|wx.FRAME_FLOAT_ON_PARENT)
        ## MYID
        # This ID is given by the parent for an instance of this class
        self.MyID = None
        # Page - the currently active page of the notebook.
        self.Page = self.parent.notebook.GetCurrentPage()
        ## Content
        self.panel = wx.Panel(self)
        self.topSizer = wx.BoxSizer(wx.VERTICAL)
        textinit = wx.StaticText(self.panel,
                    label="Create an average from the following pages:")
        self.topSizer.Add(textinit)
        ## Page selection
        self.WXTextPages = wx.TextCtrl(self.panel, value="",
                                       size=(textinit.GetSize()[0],-1))
        self.topSizer.Add(self.WXTextPages)
        ## Chechbox asking for Mono-Model
        self.WXCheckMono = wx.CheckBox(self.panel,
         label="Only use pages with the same model as the first page.")
        self.WXCheckMono.SetValue(True)
        self.topSizer.Add(self.WXCheckMono)
        ## Model selection Dropdown
        textinit2 = wx.StaticText(self.panel,
                                label="Select a model for the average:")
        self.topSizer.Add(textinit2)
        self.WXDropSelMod = wx.ComboBox(self.panel, -1, "", (15,30),
               wx.DefaultSize, [], wx.CB_DROPDOWN|wx.CB_READONLY)
        self.topSizer.Add(self.WXDropSelMod)
        textinit3 = wx.StaticText(self.panel,
         label="This tool averages only over pages with the same type"+\
               "\n(auto- or cross-correlation). Intensity data are"+\
               "\nappended sequentially.")
        self.topSizer.Add(textinit3)
        # Set all values of Text and Strin
        self.SetValues()
        btnavg = wx.Button(self.panel, wx.ID_CLOSE, 'Create average')
        # Binds the button to the function - close the tool
        self.Bind(wx.EVT_BUTTON, self.OnAverage, btnavg)
        self.topSizer.Add(btnavg)
        self.panel.SetSizer(self.topSizer)
        self.topSizer.Fit(self)
        self.SetMinSize(self.topSizer.GetMinSizeTuple())
        #Icon
        if parent.MainIcon is not None:
            wx.Frame.SetIcon(self, parent.MainIcon)
        self.Show(True)
        self.OnPageChanged(self.Page)


    def OnClose(self, event=None):
        # This is a necessary function for PyCorrFit.
        # Do not change it.
        self.parent.toolmenu.Check(self.MyID, False)
        self.parent.ToolsOpen.__delitem__(self.MyID)
        self.Destroy()


    def OnPageChanged(self, page, trigger=None):
        """
            This function is called, when something in the panel
            changes. The variable `trigger` is used to prevent this
            function from being executed to save stall time of the user.
            Forr a list of possible triggers, see the doc string of
            `tools`.
        """
        # When parent changes
        # This is a necessary function for PyCorrFit.
        # This is stuff that should be done when the active page
        # of the notebook changes.
        if trigger in ["parm_batch", "fit_batch", "page_add_batch",
                       "parm_finalize", "fit_finalize"]:
            return
        if self.parent.notebook.GetPageCount() == 0:
            self.panel.Disable()
            return
        
        #idsel = self.WXDropSelMod.GetSelection()
        self.SetValues()
        # Set back user selection:
        #self.WXDropSelMod.SetSelection(idsel)

        self.panel.Enable()
        self.Page = page


    def OnAverage(self, evt=None):
        strFull = self.WXTextPages.GetValue()
        PageNumbers = misc.parseString2Pagenum(self, strFull)
        if PageNumbers is None:
            # Something went wrong and parseString2Pagenum already displayed
            # an error message.
            return
        pages = list()
        UsedPagenumbers = list()
        # Reference page is the first page of the selection!
        #referencePage = self.parent.notebook.GetCurrentPage()
        referencePage = None
        for i in np.arange(self.parent.notebook.GetPageCount()):
            Page = self.parent.notebook.GetPage(i)
            if Page.counter.strip(" :#") == str(PageNumbers[0]):
                referencePage = Page
                break

        if referencePage is None:
            # If that did not work, we have to raise an error.
            raise IndexError("PyCorrFit could not find the first"+
							 " page for averaging.")
            return
        
        for i in np.arange(self.parent.notebook.GetPageCount()):
            Page = self.parent.notebook.GetPage(i)
            corr = Page.corr
            model = Page.corr.fit_model
            j = filter(lambda x: x.isdigit(), Page.counter)
            if int(j) in PageNumbers:
                # Get all pages with the same model?
                if self.WXCheckMono.GetValue() == True:
                    if (model.id == referencePage.corr.fit_model.id and
                       corr.is_cc == referencePage.corr.is_cc):
                        ## Check if the page has experimental data:
                        # If there is an empty page somewhere, don't bother
                        if corr.correlation is not None:
                            pages.append(Page)
                            UsedPagenumbers.append(int(j))
                else:
                    if corr.is_cc == referencePage.corr.is_cc:
                        # If there is an empty page somewhere, don't bother
                        if corr.correlation is not None:
                            pages.append(Page)
                            UsedPagenumbers.append(int(j))
        # If there are no pages in the list, exit gracefully
        if len(pages) <= 0:
            texterr_a = "At least one page with experimental data is\n"+\
                        "required for averaging. Please check the pages\n"+\
                        "that you selected for averaging."
            if self.WXCheckMono.GetValue() == True:
                texterr_a += " Note: You selected\n"+\
                 "to only use pages with same model as the first page."
            wx.MessageDialog(self, texterr_a, "Error", 
                              style=wx.ICON_ERROR|wx.OK|wx.STAY_ON_TOP)
            return
        # Now get all the experimental data
        explist = list()
        # Two components in case of Cross correlation
        tracetime = [np.array([]), np.array([])]
        tracerate = [np.array([]), np.array([])]
        TraceNumber = 0
        TraceAvailable = False # turns True, if pages contain traces
        for page in pages:
            corr = page.corr
            # experimental correlation curve
            # (at least 1d, because it might be None)
            explist.append(np.atleast_1d(1*corr.correlation))
            # trace
            # We will put together a trace from all possible traces
            # Stitch together all the traces.
            trace = corr.traces
            TraceNumber = len(trace)
            if TraceNumber > 0:
                TraceAvailable = True
                # Works with one or two traces. j = 0 or 1.
                for j in np.arange(TraceNumber):
                    if len(tracetime[j]) != 0:
                        # append to the trace
                        oldend = tracetime[j][-1]
                        # we assume that the first two points in a
                        # trace are equidistant and we will use their
                        # difference as an offset
                        offset = trace[j][:,0][1] - 2*trace[j][:,0][0]
                        newtracetime = 1.*trace[j][:,0] + offset
                        newtracetime = newtracetime + oldend
                        tracetime[j] = np.append(tracetime[j], newtracetime)
                        del newtracetime
                        tracerate[j] = np.append(tracerate[j], trace[j][:,1])
                    else:
                        # Initiate the trace
                        tracetime[j] = 1.*trace[j][:,0]
                        tracerate[j] = 1.*trace[j][:,1]
        # Now check if the length of the correlation arrays are the same:
        len0 = len(explist[0])
        for item in explist[1:]:
            if len(item) != len0:
                # print an error  message
                wx.MessageDialog(self,
                "Averaging over curves with different lengths is not"+\
                "\nsupported. When measuring, please make sure that"+\
                "\nthe measurement time for all curves is the same.",
                "Error", style=wx.ICON_ERROR|wx.OK|wx.STAY_ON_TOP)
                return
        # Now shorten the trace, because we want as little memory usage as
        # possible. I used this algorithm in read_FCS_Confocor3.py as well.
        newtraces = list()
        if TraceAvailable:
            for j in np.arange(TraceNumber):
                tracej = np.zeros((len(tracetime[j]),2))
                tracej[:,0] = tracetime[j]
                tracej[:,1] = tracerate[j]
                if len(tracej) >= 500:
                    # We want about 500 bins
                    # We need to sum over intervals of length *teiler*
                    teiler = int(len(tracej)/500)
                    newlength = len(tracej)/teiler
                    newsignal = np.zeros(newlength)
                    # Simultaneously sum over all intervals
                    for k in np.arange(teiler):
                        newsignal = \
                                newsignal+tracej[k:newlength*teiler:teiler][:,1]
                    newsignal = 1.* newsignal / teiler
                    newtimes = tracej[teiler-1:newlength*teiler:teiler][:,0]
                    if len(tracej)%teiler != 0:
                        # We have a rest signal
                        # We average it and add it to the trace
                        rest = tracej[newlength*teiler:][:,1]
                        lrest = len(rest)
                        rest = np.array([sum(rest)/lrest])
                        newsignal = np.concatenate((newsignal, rest), axis=0)
                        timerest = np.array([tracej[-1][0]])
                        newtimes = np.concatenate((newtimes, timerest), axis=0)
                    newtrace=np.zeros((len(newtimes),2))
                    newtrace[:,0] = newtimes
                    newtrace[:,1] = newsignal
                else:
                    # Declare newtrace -
                    # otherwise we have a problem down three lines ;)
                    newtrace = tracej
                newtraces.append(newtrace)
        else:
            newtraces=[None,None]
        # Everything is cleared for averaging
        exparray = np.array(explist)
        averagedata = exparray.sum(axis=0)[:,1]/len(exparray)
        # Create a copy from the first page
        average = 1*exparray[0]
        # Set average data
        average[:,1] = averagedata
        # create new page
        self.IsCrossCorrelation = self.Page.corr.is_cc
        interval = self.Page.corr.fit_ival
        # Obtain the model ID from the dropdown selection.
        idsel = self.WXDropSelMod.GetSelection()
        modelid = self.DropdownIndex[idsel]
        self.AvgPage = self.parent.add_fitting_tab(modelid = modelid,
                                                   select = True)
        self.AvgPage.corr.fit_ival = interval
        self.AvgPage.corr.correlation = average
        if self.IsCrossCorrelation is False:
            self.AvgPage.corr.corr_type = "AC average"
            newtrace = newtraces[0]
            if newtrace is not None and len(newtrace) != 0:
                self.AvgPage.corr.traces = [newtrace]
            else:
                self.AvgPage.corr.traces = []
        else:
            self.AvgPage.corr.corr_type = "CC average"
            if newtraces[0] is not None and len(newtraces[0][0]) != 0:
                self.AvgPage.corr.traces = newtraces
            else:
                self.AvgPage.corr.traces = []
        self.AvgPage.Fit_enable_fitting()
        if len(pages) == 1:
            # Use the same title as the first page
            newtabti = referencePage.tabtitle.GetValue()
        else:
            # Create a new tab title
            newtabti = "Average ["+misc.parsePagenum2String(UsedPagenumbers)+"]"
        self.AvgPage.tabtitle.SetValue(newtabti)
        # Set the addition information about the variance from averaging
        Listname = "Average"
        listname = Listname.lower()
        standarddev = exparray.std(axis=0)[:,1]
        if np.sum(np.abs(standarddev)) == 0:
            # The average sd is zero. We probably made an average
            # from only one page. In this case we do not enable
            # average weighted fitting
            pass
        else:
            # TODO:
            # kind of hackish to repeat this three times:
            #   self.AvgPage.corr.set_weights(Listname,  standarddev)
            self.AvgPage.corr.set_weights(listname,  standarddev)
            WeightKinds = self.AvgPage.Fitbox[1].GetItems()
            # Attention! Average weights and other external weights should
            # be sorted (for session saving).
            extTypes = self.AvgPage.corr._fit_weight_memory.keys()
            # TODO:
            # find acleaner solution
            extTypes.remove("none")
            extTypes.sort() # sorting
            for key in extTypes:
                try:
                    WeightKinds.remove(key)
                except:
                    pass
            LenInternal = len(WeightKinds)
            IndexAverag = extTypes.index(listname)
            IndexInList = LenInternal + IndexAverag
            for key in extTypes:
                WeightKinds += [key]
            self.AvgPage.Fitbox[1].SetItems(WeightKinds)
            self.AvgPage.Fitbox[1].SetSelection(IndexInList)
            self.AvgPage.corr.set_weights(listname,  standarddev)
            self.AvgPage.apply_parameters()
            self.AvgPage.corr.set_weights(listname,  standarddev)
        self.AvgPage.PlotAll()
        # Keep the average tool open.
        # self.OnClose()

    def SetPageNumbers(self, pagestring):
        self.WXTextPages.SetValue(pagestring)
        
    def SetValues(self, e=None):
        # Text input
        pagenumlist = list()
        for i in np.arange(self.parent.notebook.GetPageCount()):
            Page = self.parent.notebook.GetPage(i)
            pagenumlist.append(int(filter(lambda x: x.isdigit(), Page.counter)))
        valstring=misc.parsePagenum2String(pagenumlist)
        self.WXTextPages.SetValue(valstring)
        # Dropdown
        modelkeys = mdls.modeltypes.keys()
        modelkeys.sort()
        try:
            current_model = self.parent.notebook.GetCurrentPage().corr.fit_model.id
        except:
            current_model = -1
        i = 0
        DropdownList = list()
        self.DropdownIndex = list() # Contains model ids with same index
        current_index = 0
        for modeltype in modelkeys:
            for modelid in mdls.modeltypes[modeltype]:
                DropdownList.append(modeltype+": "+mdls.modeldict[modelid][1])
                self.DropdownIndex.append(str(modelid))
                if str(current_model) == str(modelid):
                    current_index = i
                i+=1
        self.WXDropSelMod.SetItems(DropdownList)
        self.WXDropSelMod.SetSelection(current_index)



# -*- coding: utf-8 -*-
"""
This file contains a T+T+3D+3D model for confocal FCS.
"""
from __future__ import division

import numpy as np

# 3D + 3D + Triplet Gauß
# Model 6043
def CF_Gxyz_gauss_3D3DTT(parms, tau):
    u""" Two-component three-dimensional free diffusion
        with a Gaussian laser profile, including two triplet components.
        The triplet factor takes into account a blinking term.
        Set *T* or *τ_trip* to 0, if no triplet component is wanted.

        particle1 = F₁/( (1+τ/τ₁) * sqrt(1+τ/(τ₁*SP²)))
        particle2 = α*(1-F₁)/( (1+τ/τ₂) * sqrt(1+τ/(τ₂*SP²)))
        triplet1 = 1 + T₁/(1-T₁)*exp(-τ/τ_trip₁)
        triplet2 = 1 + T₂/(1-T₂)*exp(-τ/τ_trip₂)
        norm = (F₁ + α*(1-F₁))²
        G = 1/n*(particle1 + particle2)*triplet1*triplet2/norm + offset

        *parms* - a list of parameters.
        Parameters (parms[i]):
        [0]  n        Effective number of particles in confocal volume
                      (n = n₁+n₂)
        [1]  τ₁       Diffusion time of particle species 1
        [2]  τ₂       Diffusion time of particle species 2
        [3]  F₁       Fraction of molecules of species 1 (n₁ = n*F₁)
                      0 <= F₁ <= 1
        [4]  SP       SP=z₀/r₀, Structural parameter,
                      describes elongation of the confocal volume
        [5]  α        Relative molecular brightness of particle
                      2 compared to particle 1 (α = q₂/q₁)
        [6]  τ_trip₁  Characteristic residence time in triplet state
        [7]  T₁       Fraction of particles in triplet (non-fluorescent) state
                      0 <= T < 1
        [8]  τ_trip₂  Characteristic residence time in triplet state
        [9]  T₂       Fraction of particles in triplet (non-fluorescent) state
                      0 <= T < 1
        [10] offset
        *tau* - lag time
    """
    n=parms[0]
    taud1=parms[1]
    taud2=parms[2]
    F=parms[3]
    SP=parms[4]
    alpha=parms[5]
    tautrip1=parms[6]
    T1=parms[7]
    tautrip2=parms[8]
    T2=parms[9]
    off=parms[10]

    particle1 = F/( (1+tau/taud1) * np.sqrt(1+tau/(taud1*SP**2)))
    particle2 = alpha**2*(1-F)/( (1+tau/taud2) * np.sqrt(1+tau/(taud2*SP**2)))
    # If the fraction of dark molecules is zero.
    if tautrip1 == 0 or T1 == 0:
        triplet1 = 1
    else:
        triplet1 = 1 + T1/(1-T1) * np.exp(-tau/tautrip1)
    if tautrip2 == 0 or T2 == 0:
        triplet2 = 1
    else:
        triplet2 = 1 + T2/(1-T2) * np.exp(-tau/tautrip2)
    # For alpha == 1, *norm* becomes one
    norm = (F + alpha*(1-F))**2

    G = 1/n*(particle1 + particle2)*triplet1*triplet2/norm + off
    return G

def get_boundaries_3D3DTT(parms):
    # strictly positive
    boundaries = [[0, np.inf]]*len(parms)
    # F
    boundaries[3] = [0,.9999999999999]
    # T
    boundaries[7] = [0,.9999999999999]
    boundaries[9] = [0,.9999999999999]
    boundaries[-1] = [-np.inf, np.inf]
    return boundaries


m_gauss_3d_3d_t_t_mix_6043 = [6043, "T+T+3D+3D",
                            u"Separate 3D diffusion + two triplet, Gauß",
                            CF_Gxyz_gauss_3D3DTT]

labels_6043  = [u"n",
                u"τ"+u"\u2081"+" [ms]",
                u"τ"+u"\u2082"+" [ms]",
                u"F"+u"\u2081", 
                u"SP",
                u"\u03b1"+" (q"+u"\u2082"+"/q"+u"\u2081"+")", 
                u"τ_trip₁ [ms]",
                u"T₁",
                u"τ_trip₂ [ms]",
                u"T₂",
                u"offset"
                ]

labels_human_readable_6043  = [
                        u"n",
                        u"τ₁ [ms]",
                        u"τ₂ [ms]",
                        u"F₁", 
                        u"SP",
                        u"\u03b1"+u" (q₂/q₁)", 
                        u"τ_trip₁ [µs]",
                        u"T₁",
                        u"τ_trip₂ [µs]",
                        u"T₂",
                        u"offset"
                            ]

values_6043 = [ 
                25,      # n
                5,       # taud1
                1000,    # taud2
                0.5,     # F
                5,       # SP
                1.0,     # alpha
                0.002,   # tautrip1
                0.01,    # T1
                0.001,   # tautrip2
                0.01,    # T2
                0.0      # offset
                ]   

values_factor_human_readable_6043 = [
                        1.,     # n
                        1.,     # taud1
                        1.,     # taud2
                        1.,     # F
                        1.,     # SP
                        1.,     # alpha
                        1000.,  # tautrip1 [µs]
                        1.,     # T1
                        1000.,  # tautrip2 [µs]
                        1.,     # T2
                        1.      # offset
                ]


valuestofit_6043 = [True, True, True, True, False, False, False, False, False, False, False]
parms_6043 = [labels_6043, values_6043, valuestofit_6043,
              labels_human_readable_6043, values_factor_human_readable_6043]


def MoreInfo_6043(parms, countrate=None):
    u"""Supplementary parameters:
        [9]  n₁ = n*F₁     Particle number of species 1
        [10] n₂ = n*(1-F₁) Particle number of species 2
    """
    # We can only give you the effective particle number
    n = parms[0]
    F1 = parms[3]
    Info = list()
    # The enumeration of these parameters is very important for
    # plotting the normalized curve. Countrate must come out last!
    Info.append([u"n\u2081", n*F1])
    Info.append([u"n\u2082", n*(1.-F1)])
    if countrate is not None:
        # CPP
        cpp = countrate/n
        Info.append(["cpp [kHz]", cpp])
    return Info


model = dict()
model["Parameters"] = parms_6043
model["Definitions"] = m_gauss_3d_3d_t_t_mix_6043
model["Supplements"] = MoreInfo_6043
model["Boundaries"] = get_boundaries_3D3DTT(values_6043)
model["Constraints"] = [[2, ">", 1], [6, "<", 1], [8, "<", 6]]

Modelarray = [model]


# -*- coding: utf-8 -*-
"""
PyCorrFit

Module tools - overlaycurves
Let the user choose which correlation curves to use.
Contains wrappers for file import and tools.
"""

try:
    from matplotlib import cm  # @UnresolvedImport
except:
    mpl_available = False
else:
    mpl_available = True
    
import numpy as np
import platform
import wx
import wx.lib.plot as plot              # Plotting in wxPython

from .. import edclasses
from .. import misc

# Menu entry name
MENUINFO = ["&Overlay curves", "Select experimental curves."]

class Wrapper_OnImport(object):
    """ Wrapper for import function.
        parent: wx.Frame
        curvedict: dictionary with curves
        onselected: external function that is called with two arguments:
                    *kept keys* and *unwanted keys* as lists referring to
                    curvedict.
        selkeys: preselected values for curves in curvedict
    """
    def __init__(self, parent, curvedict, onselected,
                 selkeys=None, labels=None):
        self.onselected = onselected
        self.parent = parent
        self.Selector = UserSelectCurves(parent, curvedict, wrapper=self,
                                         selkeys=selkeys, labels=labels)
        self.Selector.Show()
        self.Selector.MakeModal(True)
        self.Selector.Bind(wx.EVT_CLOSE, self.OnClose)
        
    def OnClose(self, event=None):
        self.Selector.MakeModal(False)
        self.Selector.Destroy()

        
    def OnResults(self, keyskeep, keysrem):
        """ Here we will close (or disable?) pages that are not wanted
            by the user. It is important that we do not close pages that
            do not contain any experimental data (Page.dataeyp is None),
            because we ignored those pages during import.
        """
        self.OnClose()
        self.onselected(keyskeep,keysrem)
        
 


class Wrapper_Tools(object):
    def __init__(self, parent):
        """
        Wrapper for tools menu. Gets curvedict from parent and starts
        curve selection. See *UserSelectCurves* class.
        """
        # parent is the main frame of PyCorrFit
        self.parent = parent
        ## MYID
        # This ID is given by the parent for an instance of this class
        self.MyID = None
        ## Wrapping
        curvedict, labels = self.GetCurvedict()
        self.labels = labels
        self.Selector = UserSelectCurves(parent, curvedict,
                                         wrapper=self, labels=labels)
        # This is necessary for parent to deselect and select the tool
        # in the tools menu.
        self.Bind = self.Selector.Bind
        if self.parent.notebook.GetPageCount() == 0:
            self.Selector.sp.Disable()


    def Disable(self):
        self.Selector.Disable()

    def Enable(self, par=True):
        self.Selector.Enable(par)
    
    def GetCurvedict(self, e=None):
        curvedict = dict()
        labels = dict()
        N = self.parent.notebook.GetPageCount()
        for i in np.arange(N):
            Page = self.parent.notebook.GetPage(i)
            key = Page.counter
            if Page.corr.correlation is not None:
                curve = 1*Page.corr.correlation_plot
                curvedict[key] = curve
                labels[key] = Page.tabtitle.GetValue()
        return curvedict, labels

        
    def OnClose(self, event=None):
        # This is a necessary function for PyCorrFit.
        # Do not change it.
        self.parent.toolmenu.Check(self.MyID, False)
        self.parent.ToolsOpen.__delitem__(self.MyID)
        self.Selector.Destroy()


    def OnPageChanged(self, page=None, trigger=None):
        """
            This function is called, when something in the panel
            changes. The variable `trigger` is used to prevent this
            function from being executed to save stall time of the user.
            Forr a list of possible triggers, see the doc string of
            `tools`.
        """
        if trigger in ["parm_batch", "fit_batch", "page_add_batch",
                       "tab_init", "tab_browse"]:
            return
        # When parent changes
        # This is a necessary function for PyCorrFit.
        # This is stuff that should be done when the active page
        # of the notebook changes.
        if self.parent.notebook.GetPageCount() == 0:
            self.Selector.SelectBox.SetItems([])
            self.Selector.sp.Disable()
        else:
            oldlabels = self.Selector.curvelabels
            self.Selector.sp.Enable()
            # Sticky behavior cleaned up in 0.7.8
            curvedict, labels = self.GetCurvedict()
            self.Selector.curvedict = curvedict
            self.Selector.labels = labels
            self.Selector.ProcessDict()
            self.labels = labels
            if oldlabels != self.Selector.curvelabels:
                self.Selector.SelectBox.SetItems(
                                              self.Selector.curvelabels)
            for i in np.arange(len(self.Selector.curvekeys)):
                self.Selector.SelectBox.SetSelection(i)
            self.Selector.OnUpdatePlot(trigger=trigger)


    def OnResults(self, keyskeep, keysrem):
        """ Here we will close (or disable?) pages that are not wanted
            by the user. It is important that we do not close pages that
            do not contain any experimental data (Page.dataeyp is None),
            because we ignored those pages during import.
        """
        if len(keysrem) == 0:
            self.OnClose()
            return
        # warn the user!
        # First make a list of all pages that need to be removed and then
        # delete those pages.
        overtext = "Keep only pages in this list?"
        textlist = ""
        for key in keyskeep:
            textlist += "- "+key+" "+self.labels[key]+"\n"
        dlg = edclasses.MyScrolledDialog(self.parent,
                                          overtext, textlist, "Warning")
        if dlg.ShowModal() == wx.ID_OK:
            N = self.parent.notebook.GetPageCount()
            pagerem = list()
            for i in np.arange(N):
                Page = self.parent.notebook.GetPage(i)
                key = Page.counter
                if keysrem.count(key) == 1:
                    pagerem.append(Page)
            for Page in pagerem:
                j = self.parent.notebook.GetPageIndex(Page)
                self.parent.notebook.DeletePage(j)
            self.OnPageChanged()
        dlg.Destroy()
        


    def OnSelectionChanged(self, keylist, trigger=None):
        if len(keylist) == 0:
            return
        # integer type list with page number
        pagelist = list()
        N = self.parent.notebook.GetPageCount()
        for i in np.arange(N):
            Page = self.parent.notebook.GetPage(i)
            key = Page.counter
            if keylist.count(key) == 1:
                pagelist.append(int(key.strip("#: ")))
        # Get open tools
        toolkeys = self.parent.ToolsOpen.keys()
        if len(toolkeys) == 0:
            return
        # Fill
        string = misc.parsePagenum2String(pagelist)
        for key in toolkeys:
            tool = self.parent.ToolsOpen[key]
            try:
                tool.SetPageNumbers(string)
            except:
                # tool does not have this function and hence does not
                # need numbers.
                pass
        
        
class UserSelectCurves(wx.Frame):
    # This tool is derived from a wx.frame.
    def __init__(self, parent, curvedict, wrapper=None, selkeys=None,
                 labels=None):
        """
        *curvedict* is a dictionary that contains the curves. Keys serve as
        identifiers in the curve selection.
        e.g.
        curvelist["#1:"] = np.array[ np.array[0.0,1], np.array[0.0,.971] ...]
        *parent* is the main frame
        *wrapper* is the object to which the chosen keys are given back. If
           it is not None, it must provide a function *OnResults*, accepting
           a list of keys as an argument.
        *selkeys* items in the list *curvedict* that are preelected.
        *labels* dictionary with same keys as *curvelist* - labels of the
           entries in the list. If none, the keys of *curvedict* will be used.        
        """
        # parent is the main frame of PyCorrFit
        self.parent = parent
        self.wrapper = wrapper
        self.curvedict = curvedict
        self.selkeys = selkeys
        self.labels = labels    # can be None
        self.curvelabels = None # filled by self.ProcessDict()
        if self.selkeys is not None:
            newselkeys = list()
            for item in self.selkeys:
                newselkeys.append(str(item))
            self.selkeys = newselkeys
        # Get the window positioning correctly
        pos = self.parent.GetPosition()
        pos = (pos[0]+100, pos[1]+100)
        wx.Frame.__init__(self, parent=self.parent, title="Overlay curves",
                 pos=pos, style=wx.DEFAULT_FRAME_STYLE|wx.FRAME_FLOAT_ON_PARENT,
                 size=(800,500))
        ## Pre-process
        self.ProcessDict()
        ## Content
        self.sp = wx.SplitterWindow(self, size=(500,500), style=wx.SP_NOBORDER)
        self.sp.SetMinimumPaneSize(1)
        # Top panel
        panel_top = wx.Panel(self.sp, size=(500,200))
        self.upperSizer = wx.BoxSizer(wx.VERTICAL)
        if platform.system().lower() == 'darwin':
            ctrl = "Apple"
        else:
            ctrl = "Ctrl"
        text = "By holding down the '"+ctrl+"' key, single curves can be \n" +\
               "selected or deselected. The 'Shift' key can be used \n" +\
               "to select groups."
        self.upperSizer.Add(wx.StaticText(panel_top, label=text))
        # Bottom Panel
        self.bottom_sp = wx.SplitterWindow(self.sp, size=(500,300), style=wx.SP_NOBORDER)
        self.bottom_sp.SetMinimumPaneSize(1)
        sizepanelx = 250
        panel_bottom = wx.Panel(self.bottom_sp, size=(sizepanelx,300))
        self.boxSizer = wx.BoxSizer(wx.VERTICAL)
        # Box selection
        style = wx.LB_EXTENDED
        self.SelectBox = wx.ListBox(panel_bottom, size=(sizepanelx,300),
                                    style=style, choices=self.curvelabels)
        for i in np.arange(len(self.curvekeys)):
            self.SelectBox.SetSelection(i)

        # Deselect keys that are not in self.selkeys
        if self.selkeys is not None:
            for i in np.arange(len(self.curvekeys)):
                if self.selkeys.count(self.curvekeys[i]) == 0:
                    self.SelectBox.Deselect(i)
        self.Bind(wx.EVT_LISTBOX, self.OnUpdatePlot, self.SelectBox)
        self.boxSizer.Add(self.SelectBox, wx.EXPAND)
        minsx = self.boxSizer.GetMinSize()[0]
        # Button REMOVE
        btnrem = wx.Button(panel_bottom, wx.ID_ANY, 'Remove selected')
        self.Bind(wx.EVT_BUTTON, self.OnPushResultsRemove, btnrem)
        btnrem.SetMinSize((minsx, -1))
        self.boxSizer.Add(btnrem)
        # Button KEEP
        btnkep = wx.Button(panel_bottom, wx.ID_ANY, 'Keep selected')
        self.Bind(wx.EVT_BUTTON, self.OnPushResultsKeep, btnkep)
        self.boxSizer.Add(btnkep)
        btnkep.SetMinSize((minsx, -1))
        # Button CANCEL
        btncancel = wx.Button(panel_bottom, wx.ID_ANY, 'Cancel')
        self.Bind(wx.EVT_BUTTON, self.OnCancel, btncancel)
        self.boxSizer.Add(btncancel)
        btncancel.SetMinSize((minsx, -1))
        # Finish off sizers
        panel_top.SetSizer(self.upperSizer)
        panel_bottom.SetSizer(self.boxSizer)
        self.upperSizer.Fit(panel_top)
        self.boxSizer.Fit(panel_bottom)
        minsize = np.array(self.boxSizer.GetMinSizeTuple()) +\
                  np.array(self.upperSizer.GetMinSizeTuple()) +\
                  np.array((300,30))
        self.SetMinSize(minsize)
        #self.SetSize(minsize)
        #self.SetMaxSize((9999, self.boxSizer.GetMinSizeTuple()[1]))
        # Canvas
        self.canvas = plot.PlotCanvas(self.bottom_sp)
        self.canvas.setLogScale((True, False))  
        self.canvas.SetEnableZoom(True)
        # Splitter window
        self.bottom_sp.SplitVertically(panel_bottom, self.canvas, sizepanelx)
        sizetoppanel = self.upperSizer.GetMinSizeTuple()[1]
        self.sp.SplitHorizontally(panel_top, self.bottom_sp, sizetoppanel)
        self.OnUpdatePlot()
        # Icon
        if parent.MainIcon is not None:
            wx.Frame.SetIcon(self, parent.MainIcon)
        self.Show(True)

    
    def GetSelection(self):
        keyssel = list()
        for i in self.SelectBox.GetSelections():
            keyssel.append(self.curvekeys[i])
        keysnosel = list()
        for key in self.curvekeys:
            if keyssel.count(key) == 0:
                keysnosel.append(key)
        return keyssel, keysnosel
    
    
    def ProcessDict(self, e=None):
        # Define the order of keys used.
        # We want to sort the keys, such that #10: is not before #1:
        self.curvekeys = self.curvedict.keys()
        # Sorting key function applied to each key before sorting:
        page_num = lambda counter: int(counter.strip().strip(":").strip("#"))
        try:
            for item in self.curvekeys:
                page_num(item)
        except:
            fstr = lambda x: x
        else:
            fstr = page_num
        self.curvekeys.sort(key = fstr)
        if self.labels is None:
            self.curvelabels = self.curvekeys
        else:
            # Use given labels instead of curvekeys.
            self.curvelabels = list()
            for key in self.curvekeys:
                self.curvelabels.append("#"+str(key).strip(":# ")+" "+self.labels[key])


    def OnCancel(self, e=None):
        """ Close the tool """
        self.wrapper.OnClose()
        

    def OnPushResultsRemove(self, e=None):
        # Get keys from selection
        keysrem, keyskeep = self.GetSelection()
        self.wrapper.OnResults(keyskeep, keysrem)


    def OnPushResultsKeep(self, e=None):
        # Get keys from selection
        keyskeep, keysrem = self.GetSelection()
        self.wrapper.OnResults(keyskeep, keysrem)


    def OnUpdatePlot(self, e=None, trigger=None):
        """ What should happen when the selection in *self.SelectBox*
            is changed?
            
            This function will alsy try to call the function
            *self.parent.OnSelectionChanged* and hand over the list of
            currently selected curves. This is an addon for 0.7.8
            where we will control the page selection in the average
            tool.
            If `trigger` is something that occurs during loading of
            data, then we will not replot everything.
        """
        #if e is not None and e.GetEventType() == 10007:
        #    return

        # Get selected curves
        curves = list()
        legends = list()
        selection = self.SelectBox.GetSelections()
        for i in selection:
            curves.append(self.curvedict[self.curvekeys[i]])
            legends.append(self.curvekeys[i])
        # Set color map
        if mpl_available:
            cmap = cm.get_cmap("gist_rainbow")
        # Clear Plot
        self.canvas.Clear()
        # Draw Plot
        lines = list()
        for i in np.arange(len(curves)):
            if mpl_available:
                color = cmap(1.*i/(len(curves)), bytes=True)
                color = wx.Colour(color[0], color[1], color[2])
            else:
                color = "black"
            line = plot.PolyLine(curves[i], legend=legends[i],
                                 colour=color, width=1)
            lines.append(line)
        self.canvas.SetEnableLegend(True)
        if len(curves) != 0:
            self.canvas.Draw(plot.PlotGraphics(lines, 
                         xLabel=u'lag time τ [ms]', 
                         yLabel=u'G(τ)'))
        ## This is an addon for 0.7.8
        keyskeep = list()
        for i in self.SelectBox.GetSelections():
            keyskeep.append(self.curvekeys[i])
        try:
            self.wrapper.OnSelectionChanged(keyskeep, trigger=trigger)
        except:
            pass



# -*- coding: utf-8 -*-
"""
PyCorrFit

Module wxutils
"""
import numpy as np                      # NumPy
import re
import string
import wx                               # GUI interface wxPython


def float2string_nsf(fval, n=7):
    """
    Truncate a float to n significant figures and return nice string.
    Arguments:
      q : a float
      n : desired number of significant figures
    Returns:
    String with only n s.f. and trailing zeros.
    """
    #sgn=np.sign(fval)
    try:
        if fval == 0:
            npoint=n
        else:
            q=abs(fval)
            # prevent k from having negative values
            k=max(0,int(np.ceil(np.log10(q/n))))
            npoint = n-k
        string="{:.{}f}".format(fval, npoint)
    except:
        string="{}".format(fval)
    return string

def nice_string(string):
    """
    Convert a string of a float created by `float2string_nsf`
    to something nicer.
    
    i.e.
    - 1.000000 -> 1
    - 1.010000 -> 1.010
    """
    if string.count(".") and len(string.split(".")[1].replace("0", "")) == 0:
        return "{:d}".format(int(float(string)))
    else:
        olen = len(string)
        newstring = string.rstrip("0")
        if olen > len(newstring):
            string=newstring+"0"
        return string


class PCFFloatValidator(wx.PyValidator):
    def __init__(self, flag=None, pyVar=None):
        wx.PyValidator.__init__(self)
        self.flag = flag
        self.Bind(wx.EVT_CHAR, self.OnChar)

    def Clone(self):
        return PCFFloatValidator(self.flag)

    def Validate(self, win):
        tc = self.GetWindow()
        val = tc.GetValue()
        
        for x in val:
            if x not in string.digits:
                return False

        return True

    def OnChar(self, event):
        """
        Filter the characters that are put in the control.
        
        TODO:
        - check for strings that do not make sense
          - 2e-4.4
          - 2e--3
          - 3-1+5
        """
        key = event.GetKeyCode()
        ctrl = event.GetEventObject()
        # Get the actual string from the object
        curval = wx.TextCtrl.GetValue(ctrl)

        if key < wx.WXK_SPACE or key == wx.WXK_DELETE or key > 255:
            event.Skip()
            return

        char = chr(key)
        char = char.replace(",", ".")
        
        onlyonce = [".", "e", "i", "n", "f"]
        if char in onlyonce and curval.count(char):
            # not allowed
            return

        if ( char in string.digits or
             char in ["+", "-"]+onlyonce):
            event.Skip()
            return

        if not wx.Validator_IsSilent():
            wx.Bell()

        # Returning without calling event.Skip eats the event before it
        # gets to the text control
        return


class PCFFloatTextCtrl(wx.TextCtrl):
    def __init__(self, *args, **kwargs):
        wx.TextCtrl.__init__(self, *args, validator=PCFFloatValidator(), size=(110,-1),
                             style=wx.TE_PROCESS_ENTER, **kwargs)
        self.Bind(wx.EVT_ENTER_WINDOW, self.OnMouseEnter)
        self.Bind(wx.EVT_LEAVE_WINDOW, self.OnMouseLeave)
        self._PCFvalue = 0.0

    def OnMouseEnter(self, e):
        self.SetFocus()
        self.SetSelection(-1,0)

    def OnMouseLeave(self, e):
        self.SetSelection(0,0)
        self.SetInsertionPoint(0)

    def SetValue(self, value):
        self._PCFvalue = value
        string = PCFFloatTextCtrl.float2string(value)
        wx.TextCtrl.SetValue(self, string)
    
    def GetValue(self):
        string = wx.TextCtrl.GetValue(self)
        if string == PCFFloatTextCtrl.float2string(self._PCFvalue):
            # use internal value: more accurate
            #print("internal", self._PCFvalue)
            return self._PCFvalue
        else:
            # new value
            #print("external", string)
            return PCFFloatTextCtrl.string2float(string)
        
    @staticmethod
    def float2string(value):
        """
        inverse of string2float with some tweaks
        """
        value = float2string_nsf(value)
        value = nice_string(value)
        return value
        
    @staticmethod
    def string2float(string):
        """
        Remove any characters that are not in
        [+-{0-9},.] and show a decent float
        value.
        """
        if string.count("inf"):
            if string[0]=="-":
                return -np.inf
            else:
                return np.inf
        # allow comma
        string = string.replace(",", ".")
        # allow only one decimal point
        string = string[::-1].replace(".", "", string.count(".")-1)[::-1]
        try:
            string = "{:.12f}".format(float(string))
        except:
            pass
        # remove letters
        string = re.sub(r'[^\d.-]+', '', string)
        if len(string) == 0:
            string = "0"
        return float(string)



# -*- coding: utf-8 -*-
"""
This file contains 3D+3D TIR-FCS models.
"""
import numpy as np                  # NumPy
import scipy.special as sps


def wixi(x):
    """ Complex Error Function (Faddeeva/Voigt).
        w(i*x) = exp(x**2) * ( 1-erf(x) )
        This function is called by other functions within this module.
        We are using the scipy.special.wofz module which calculates
        w(z) = exp(-z**2) * ( 1-erf(-iz) )
        z = i*x
    """
    z = x*1j
    wixi = sps.wofz(z)
    # We should have a real solution. Make sure nobody complains about
    # some zero-value imaginary numbers.
    
    return np.real_if_close(wixi)


# 3D + 3D no binding TIRF
# model 6023
def CF_Gxyz_TIR_square_3d3d(parms, tau, wixi=wixi):
    u""" Two-component three-dimensional free diffusion
        with a square-shaped lateral detection area taking into account
        the size of the point spread function; and an exponential
        decaying profile in axial direction.

        *parms* - a list of parameters.
        Parameters (parms[i]):
        [0] D_3D1  3D Diffusion coefficient (species 1)
        [1] D_3D2  3D Diffusion coefficient of bound species 2
        [2] σ      Lateral size of the point spread function
                   σ = σ₀ * λ / NA
        [3] a      Side size of the square-shaped detection area
        [4] d_eva  Evanescent penetration depth
        [5] C_3D1  Concentration of species 1
        [6] C_3D2  Concentration of species 2
        [7] α      Relative molecular brightness of particle
                   2 compared to particle 1 (α = q₂/q₁)
        *tau* - lag time
    """
    D_3D1 = parms[0]
    D_3D2 = parms[1]
    sigma = parms[2]
    a = parms[3]
    kappa = 1/parms[4]
    Conc_3D1 = parms[5]
    Conc_3D2 = parms[6]
    alpha = parms[7]

    ## First, the 3D diffusion of species 1
    # Axial correlation    
    x1 = np.sqrt(D_3D1*tau)*kappa
    w_ix1 = wixi(x1)
    gz1 = np.sqrt(D_3D1*tau/np.pi) - (2*D_3D1*tau*kappa**2 - 1)/(2*kappa) * \
          w_ix1
    # Lateral correlation
    gx1_1 = 2/(a**2*np.sqrt(np.pi)) * np.sqrt(sigma**2+D_3D1*tau) * \
            ( np.exp(-a**2/(4*(sigma**2+D_3D1*tau))) -1 )
    gx2_1 = 1/a * sps.erf( a / (2*np.sqrt(sigma**2 + D_3D1*tau))) 
    gx1 =  gx1_1 + gx2_1
    gxy1 = gx1**2
    # Non normalized correlation function
    g3D1 = Conc_3D1 * gxy1 * gz1

    ## Second, the 3D diffusion of species 2
    # Axial correlation    
    x2 = np.sqrt(D_3D2*tau)*kappa
    w_ix2 = wixi(x2)
    gz2 = np.sqrt(D_3D2*tau/np.pi) - (2*D_3D2*tau*kappa**2 - 1)/(2*kappa) * \
          w_ix2
    # Lateral correlation
    gx1_2 = 2/(a**2*np.sqrt(np.pi)) * np.sqrt(sigma**2+D_3D2*tau) * \
            ( np.exp(-a**2/(4*(sigma**2+D_3D2*tau))) -1 )
    gx2_2 = 1/a * sps.erf( a / (2*np.sqrt(sigma**2 + D_3D2*tau))) 
    gx2 =  gx1_2 + gx2_2
    gxy2 = gx2**2
    # Non normalized correlation function
    g3D2 = alpha**2 * Conc_3D2 * gxy2 * gz2

    ## Finally the Prefactor
    F = (Conc_3D1 + alpha * Conc_3D2) / kappa
    G = (g3D1 + g3D2) / F**2
    return G


# 3D-3D Model TIR
m_tir_3d_3d_mix_6023 = [6023, u"3D+3D",
                        "Separate 3D diffusion, 3D TIR",
                        CF_Gxyz_TIR_square_3d3d]
labels_6023 = [u"D"+u"\u2081"+u" [10 µm²/s]",
               u"D"+u"\u2082"+u" [10 µm²/s]",
               u"σ [100 nm]",
               u"a [100 nm]", 
               u"d_eva [100 nm]", 
               u"C"+u"\u2081"+u" [1000 /µm³]", 
               u"C"+u"\u2082"+u" [1000 /µm³]", 
               u"\u03b1"+" (q"+u"\u2082"+"/q"+u"\u2081"+")"
                ]
values_6023 = [
                0.9,     # D_3D₁ [10 µm²/s]
                9.0,     # D_3D₂ [10 µm²/s]
                2.3,     # σ [100 nm]
                7.50,    # a [100 nm]
                1.0,     # d_eva [100 nm]
                0.01,    # conc.3D₁ [1000 /µm³]
                0.03,    # conc.3D₂ [1000 /µm³]
                1        # alpha
                ]        
# For user comfort we add values that are human readable.
# Theese will be used for output that only humans can read.
labels_human_readable_6023 = [
                u"D"+u"\u2081"+u" [µm²/s]",
                u"D"+u"\u2082"+u" [µm²/s]",
                u"σ [nm]",
                u"a [nm]", 
                u"d_eva [nm]", 
                u"C"+u"\u2081"+u" [1/µm³]", 
                u"C"+u"\u2082"+u" [1/µm³]", 
                u"\u03b1"+" (q"+u"\u2082"+"/q"+u"\u2081"+")"
                ]
values_factor_human_readable_6023 = [10, # "D_3D₁ [µm²/s]",
                10,     # D_3D₂ [10 µm²/s]
                100,    # σ [100 nm]
                100,    # a [100 nm]
                100,    # d_eva [100 nm]
                1000,   # conc.3D₁ [1000 /µm³]
                1000,   # conc.3D₂ [1000 /µm³]
                1       # alpha
                ]
valuestofit_6023 = [False, True, False, False, False, False, True, False]
parms_6023 = [labels_6023, values_6023, valuestofit_6023, 
              labels_human_readable_6023, values_factor_human_readable_6023]


model1 = dict()
model1["Parameters"] = parms_6023
model1["Definitions"] = m_tir_3d_3d_mix_6023
model1["Boundaries"] = [[0, np.inf]] * len(values_6023)
model1["Constraints"] = [[1, ">", 0]]

Modelarray = [model1]



# -*- coding: utf-8 -*-
u"""  PyCorrFit  - module "readfiles"

Import correlation data from data files.
"""
# This file is necessary for this folder to become a module that can be 
# imported by PyCorrFit.

import csv
import numpy as np
import os
import sys
import tempfile
import yaml
import warnings
import zipfile

# To add a filetype add it here and in the
# dictionaries at the end of this file.
from .read_ASC_ALV import openASC
from .read_CSV_PyCorrFit import openCSV
from .read_SIN_correlator_com import openSIN
from .read_FCS_Confocor3 import openFCS
from .read_mat_ries import openMAT
from .read_pt3_PicoQuant import openPT3



def AddAllWildcard(Dictionary):
    wildcard = ""
    keys = Dictionary.keys()
    N = len(keys)
    i = 0
    for key in keys:
        newwc = key.split("|")[1]
        wildcard = wildcard + newwc
        i = i + 1
        if i != N:
            wildcard = wildcard + ";"

    Dictionary[Allsupfilesstring+"|"+wildcard] = openAny
    return Dictionary


# To increase user comfort, we will now create a file opener thingy that
# knows how to open all files we know.
def openAny(dirname, filename):
    """ Using the defined Filetypes and BGFiletypes, open the given file """
    wildcard = filename.split(".")[-1]
    for key in Filetypes.keys():
        # Recurse into the wildcards
        wildcardstring = key.split("|")
        # We do not want to recurse
        if wildcardstring[0] != Allsupfilesstring:
            otherwcs = wildcardstring[1].split(";")
            for string in otherwcs:
                if string.strip(" .*") == wildcard:
                    return Filetypes[key](dirname, filename)
    # If we could not find the correct function in Filetypes, try again
    # in BGFiletypes:
    return openAnyBG(dirname, filename)
    
    ## For convenience in openZIP
    #return None # already in openAnyBG


def openAnyBG(dirname, filename):
    wildcard = filename.split(".")[-1]
    for key in BGFiletypes.keys():
        wildcardstring = key.split("|")
        # We do not want to recurse
        if wildcardstring[0] != Allsupfilesstring:
            otherwcs = wildcardstring[1].split(";")
            for string in otherwcs:
                if string.strip(" .*") == wildcard:
                    return BGFiletypes[key](dirname, filename)
    # For convenience in openZIP
    return None
    

def openZIP(dirname, filename):
    """ 
        Get everything inside a .zip file that could be an FCS curve.
        Will use any wildcard in Filetypes dictionary.
    """
    #    It's a rather lengthy import of the session file. The code is copied
    #    from openfile.OpenSession. The usual zip file packed curves are
    #    imported on the few code lines after the else statement.

    ## Open the archive:
    Arc = zipfile.ZipFile(os.path.join(dirname, filename), mode='r')
    Correlations = list() # Correlation information
    Curvelist = list()    # Type information
    Filelist = list()     # List of filenames corresponding to *Curvelist*
    Trace = list()        # Corresponding traces
    ## First test, if we are opening a session file
    sessionwc = [".fcsfit-session.zip", ".pcfs"]
    if ( filename.endswith(sessionwc[0]) or
         filename.endswith(sessionwc[1])     ):
        # Get the yaml parms dump:
        yamlfile = Arc.open("Parameters.yaml")
        # Parms: Fitting and drawing parameters of the correlation curve
        # The *yamlfile* is responsible for the order of the Pages #i.
        # The parameters are actually useless to us right now.
        Parms = yaml.safe_load(yamlfile)
        yamlfile.close()
        # Get the correlation arrays
        ImportedNum = list()
        for i in np.arange(len(Parms)):
            # The *number* is used to identify the correct file
            number = str(Parms[i][0])
            expfilename = "data"+number[1:len(number)-2]+".csv"
            expfile = Arc.open(expfilename, 'r')
            readdata = csv.reader(expfile, delimiter=',')
            dataexp = list()
            if str(readdata.next()[0]) == "# tau only":
                # We do not have a curve here
                pass
            else:
                Filelist.append(filename+"/#"+number[1:len(number)-2])
                for row in readdata:
                    # Exclude commentaries
                    if (str(row[0])[0:1] != '#'):
                        dataexp.append((float(row[0]), float(row[1])))
                dataexp = np.array(dataexp)
                Correlations.append(dataexp)
                ImportedNum.append(i)
            del readdata
            expfile.close()
        # Get the Traces
        for i in ImportedNum:   
            # Make sure we only import those traces that had a corresponding
            # correlation curve. (ImportedNum)
            #
            # The *number* is used to identify the correct file
            number = str(Parms[i][0])
            # Find out, if we have a cross correlation data type
            IsCross = False
            try:
                IsCross = Parms[i][7]
            except IndexError:
                # No Cross correlation
                pass
            if IsCross is False:
                tracefilenames = ["trace"+number[1:len(number)-2]+".csv"]
                Curvelist.append("AC")
            else:
                # Cross correlation uses two traces
                tracefilenames = ["trace"+number[1:len(number)-2]+"A.csv",
                                  "trace"+number[1:len(number)-2]+"B.csv" ]
                Curvelist.append("CC")
            thistrace = list()
            for tracefilename in tracefilenames:
                try:
                    Arc.getinfo(tracefilename)
                except KeyError:
                    # No correlation curve, but add a None
                    pass
                else:
                    tracefile = Arc.open(tracefilename, 'r')
                    traceread = csv.reader(tracefile, delimiter=',')
                    singletrace = list()
                    for row in traceread:
                        # Exclude commentaries
                        if (str(row[0])[0:1] != '#'):
                            singletrace.append((float(row[0]), float(row[1])))
                    singletrace = np.array(singletrace)
                    thistrace.append(singletrace)
                    del traceread
                    del singletrace
                    tracefile.close()
            if len(thistrace) == 1:
                Trace.append(thistrace[0])
            elif len(thistrace) == 2:
                Trace.append(thistrace)
            else:
                Trace.append(None)
    else:
        # We are not importing from a session but from a zip file with
        # probably a mix of all filetypes we know. This works 
        # recursively (e.g. a zip file in a zipfile).
        allfiles = Arc.namelist()
        # Extract data to temporary folder
        tempdir = tempfile.mkdtemp()
        rmdirs = list()
        for afile in allfiles:
            apath = Arc.extract(afile, path=tempdir)
            if os.path.isdir(apath):
                rmdirs.append(apath)
                continue
            ReturnValue = openAny(tempdir, afile)
            if ReturnValue is not None:
                Correlations += ReturnValue["Correlation"]
                Trace += ReturnValue["Trace"]
                Curvelist += ReturnValue["Type"]
                fnames = ReturnValue["Filename"]
                Filelist += [ filename+"/"+fs for fs in fnames ]
            # Delte file
            try:
                os.remove(os.path.join(tempdir, afile))
            except:
                warnings.warn("{}".format(sys.exc_info()[1]))
        for rmd in rmdirs:
            try:
                os.removedirs(rmd)
            except:
                    warnings.warn("{}".format(sys.exc_info()[1]))
        try:
            os.removedirs(tempdir)
        except:
                warnings.warn("{}".format(sys.exc_info()[1]))
    Arc.close()
    dictionary = dict()
    dictionary["Correlation"] = Correlations
    dictionary["Trace"] = Trace
    dictionary["Type"] = Curvelist
    dictionary["Filename"] = Filelist
    return dictionary


# The string that is shown when opening all supported files
# We add an empty space so it is listed first in the dialogs.
Allsupfilesstring = " All supported files"

# Dictionary with filetypes that we can open
# The wildcards point to the appropriate functions.
Filetypes = { "Correlator.com (*.SIN)|*.SIN;*.sin" : openSIN,
              "ALV (*.ASC)|*.ASC;*.asc" : openASC,
              "PyCorrFit (*.csv)|*.csv" : openCSV,
              "Matlab 'Ries (*.mat)|*.mat" : openMAT,
              "PicoQuant (*.pt3)|*.pt3" : openPT3,
              "Zeiss ConfoCor3 (*.fcs)|*.fcs" : openFCS,
              "Zip file (*.zip)|*.zip" : openZIP,
              "PyCorrFit session (*.pcfs)|*.pcfs" : openZIP
            }
# For user comfort, add "All supported files" wildcard:
Filetypes = AddAllWildcard(Filetypes)


# Dictionary with filetypes we can open that have intensity traces in them.
BGFiletypes = { "Correlator.com (*.SIN)|*.SIN;*.sin" : openSIN,
                "ALV (*.ASC)|*.ASC" : openASC,
                "PyCorrFit (*.csv)|*.csv" : openCSV,
                "PicoQuant (*.pt3)|*.pt3" : openPT3,
                "Zeiss ConfoCor3 (*.fcs)|*.fcs" : openFCS,
                "Zip file (*.zip)|*.zip" : openZIP,
                "PyCorrFit session (*.pcfs)|*.pcfs" : openZIP
              }
BGFiletypes = AddAllWildcard(BGFiletypes)

